#!/usr/bin/env python3
# automome.py
# automome - the OTTish AUTOMEME clone
# Copyright (C) 2014  Penguin Development
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
import copy
import getopt
import itertools
import os
import os.path
import random
import re
import string
import sys

# Noun index specification:
# 0: singular w/ article
# 1: plural
n_singular, n_plural = range(2)

# Verb index specification:
# 0: infinitive
# 1: continuous
# 2: present
# 3: past
# 4: perfect
v_inf, v_cont, v_present, v_past, v_perfect = range(5)

# Adjective index specification:
# Positive w/article
# Comparative w/article
# Superlative
# Propetry ("x-ness")
a_pos, a_comp, a_super, a_property = range(4)

# Currying lambda:
curry = lambda a, b: lambda x: a(b(x))

# Stringification lambda:
s = lambda a: curry(a, str)

def skipuntilvowel(s):
    """Function that returns the given string from the first vowel onwards.
    Returns the whole string if it has no vowels."""
    s = str(s)
    mat = re.compile(r'[AEIOU]').search(s)
    return s[mat.start():] if mat else s

# Lambda that turns a verb of form [x]ING into [x]ER:
erify_s = lambda x: (x[:-4] + 'IER') if x.endswith('YING') else (x[:-3] + 'ER')
erify = s(erify_s)

# Lambda that turns `THE [x]' into just `[x]':
skipthe_s = lambda x: x[4:] if x.startswith('THE ') else x
skipthe = s(skipthe_s)

# Lambda that skips the first word:
skipword = lambda x: ' '.join(str(x).split()[1:])

# Lambda that strips the indefinite article (A/AN) from singular nouns and
# adjectives:
skipart = lambda x: ' '.join(str(x).split()[:-1]) if str(x).endswith(' ИA') \
        else (skipword(x) if (x.originclass.startswith('adjective') \
        and x.wordform != a_super) or (x.originclass.startswith('noun') \
        and x.wordform == n_singular) else x.stringform)

# Lambda that strips the indefinite article from mass nouns, unique nouns
# and properties only:
skipart_mup = lambda x: (' '.join(str(x).split()[:-1]) \
        if str(x).endswith(' ИA') else skipword(x)) if x.originclass in \
        ('noun-mass', 'noun-unique') or (x.originclass == 'adjective' \
        and x.wordform == a_property) else x.stringform

# Lambda that prepends 'THE ' to words from the noun-unique class:
uniquethe = lambda x: ('THE ' + skipword(x.stringform)) \
        if x.originclass == 'noun-unique' else skipart_mup(x)

# Lambda to convert a word into Elmer Fudd speech:
fudd = lambda x: str(x).replace('R', 'W').replace('L', 'W')

# Letter multipliers by mrob27
aggile = lambda x: re.sub(r'([AEIOU])([^AEIOU])', r'\1\2\2', str(x), 1)
aaaagile = lambda x: re.sub(r'([AEIOU])', r'\1\1\1\1', str(x), 1)
aggiiiile = lambda x: re.sub(r'([AEIOU])([^AEIOU])([^AEIOU]*)([AEIOU])',
        r'\1\2\2\3\4\4\4\4', str(x), 1)

def dawgify(x):
    """Dawgify function by azule and mrob27."""
    rex = ((r'^THE ', r''), (r'O\b', 'OH'), (r'\bA|A\b', 'UH'),
            (r'EA', r'EE'), (r'X', r'KS'), (r'CK', r'KK'), (r'C|CH', r'K'),
            (r'UR', r'R'), (r'IR', r'UR'), (r'ER', r'A'), (r'TH', r'D'),
            (r'EW', r'U'), (r'CU', r'KYU'), (r'EST', r'IST'),
            (r'([^AEIOU])\1', r'\1'), (r'Y\b', r'EE'), (r'WH', r'W'),
            (r'I([^AEIOU])E\b', r'IE\1'), (r'O([^AEIOU])E\b', r'OA\1'))
    x = str(x)
    for w, repl in rex:
        x = re.sub(w, repl, x)
    return x

class WordChoice(object):
    """Class specifying a chosen word. Objects of this type should not be
    modified by any user-defined functions."""
    wordforms = None
    wordform = 0
    originclass = None
    stringform = None

    def __str__(self):
        """Method to generate the canonical string representation of this word
        choice. This method will return `stringform' unless it is None, in
        which case wordforms[wordform] is returned instead. If `wordforms' is
        itself None, the empty string is returned, although this should not
        normally happen."""
        if self.stringform is not None:
            return self.stringform
        elif self.wordforms is not None:
            return self.wordforms[self.wordform]
        else:
            return ''

class Replaceable(object):
    """Replaceable class. Objects of this class generate words to fill the
    meme templates."""
    scandir = '.'
    classes = {}

    def __init__(self, cls, defaults=None, selector=None, postproc=None):
        """Constructor.

        `defaults' is a list of tuples of default words and relative
        probabilities; e.g. [('FOO', 0.5), ('BAR', 0.33)] means there is first
        a 50% chance of 'FOO' being used, failing that, a 33% chance of 'BAR'
        being used, and failing that, a 100% chance of normal selection being
        applied.

        `cls' is a list of classes from which a replacement word may be
        chosen; an entry for a multiword class may also be given as a tuple of
        the class name and an index; e.g. ['noun-mass', ('noun', 1)]
        would pick from mass-nouns and plural nouns (since inex 1 for
        multiform nouns is the plural form).

        `selector' is a predicate function that can be used to pre-filter
        acceptable words. Predicate functions should take one argument: a
        WordChoice object specifying the word slated for selection. The
        `stringform' attribute of the WordChoice object will be the default
        form, i.e. wordforms[wordform]. The predicate should return True if
        this choice is acceptable, or False if it isn't.  Words for which the
        predicate returns False are removed from the list of possible choices.
        If the predicate does not return True for any of the choices, the
        empty string is chosen. If `selector' is None, all words from the
        classes listed in `cls' are deemed acceptable.

        `postproc' is a postprocessing function taking one argument: a
        WordChoice object specifying the word slated for selection. The
        `stringform' attribute of the WordChoice object will be set to the
        default form of the word. The postprocessing function should return
        the desired new final form of the word. The postprocessing function
        must not modify the WordChoice object. If `postproc' is None, the word
        will be used as-is."""
        self.defaults = defaults
        self.selector = selector
        self.postproc = postproc
        self.scanclasses()
        if cls is None:
            self.cls = None
            return

        self.acceptlist = None

        self.cls = []
        for c in cls:
            if isinstance(c, str):
                if c not in self.classes:
                    raise KeyError(
                            'Mustard: word class `{}\' not found!'.format(c))
                self.cls.append((c, 0))
            else:
                if c[0] not in self.classes:
                    raise KeyError(
                            'Mustard: word class `{}\' not found!'.format(
                                c[0]))
                self.cls.append(tuple(c))

    @classmethod
    def scanclasses(cls):
        """Scans the directory Replaceable.scandir for files named *.dict.txt.
        The files' base names (i.e. the part before .dict.txt) are used as
        names for word classes; the files contain one word per line, or
        multiple word forms separated by the pipe symbol | if the file name
        ends in .dict.txt.  Whitespace is stripped off the start and
        end of each word form, and empty lines (after stripping) and lines
        starting with # are ignored.

        This method aborts immediately if it has been called previously; set
        Replaceable.classes to the empty dictionary before calling this method
        to force a rescan."""

        if len(cls.classes) != 0:
            return

        for fn in os.listdir(cls.scandir):
            ffn = os.path.join(cls.scandir, fn)
            if fn.endswith('.dict.txt') and os.path.isfile(ffn):
                f = open(ffn, 'r', encoding='UTF-8')
                # The encoding in the line above is to prevent CGI mustard.
                data = [x.strip() for x in f.read().splitlines()]
                f.close()
                data = [d for d in data if d and not d.startswith('#')]
                if len(data):
                    cls.classes[fn[:-9]] = [[y.strip() for y in
                        x.split('|')] for x in data]

        cls.classes['adjective'] = cls.classes['adjective-positive'] \
            + cls.classes['adjective-negative'] \
            + cls.classes['adjective-neutral']

    def pick(self):
        """Word-picking method. Returns a WordChoice object specifying the
        chosen word. This method will build and cache a list of words that are
        acceptable according to self.selector."""
        c = WordChoice()
        c.originclass = 'x-defaults'
        # Try picking from the probability-weighted defaults first.
        if self.defaults is not None:
            for x in self.defaults:
                if random.random() > (1. - float(x[1])):
                    c.stringform = x[0]
                    return c

        # If the default-picking routine above did not pick a word, and this
        # Replaceable does not have any word classes, pick a random default,
        # or the empty string if no defaults are available.
        if self.cls is None or len(self.cls) == 0:
            c.stringform = '' if self.defaults is None else \
                    random.choice(self.defaults)[0]
            return c

        # If there is no list of acceptable words yet, generate it now from
        # the list of classes. The acceptable word list contains tuples of the
        # format
        #   (class, wordform, index)
        # with class a name in self.classes, wordform the column index in the
        # class definition file, and index the row index in the class
        # definition file.
        if self.acceptlist is None:
            self.acceptlist = []
            for cls in self.cls:
                self.acceptlist += [(cls[0], i, j) for i in cls[1:] for j in
                        range(len(self.classes[cls[0]]))]

        # This loop grabs a random word from self.acceptlist and tests it
        # against self.selector. If the selection succeeds, the loop exits; if
        # it fails, the word is removed from self.acceptlist to speed up
        # further selection attempts, and the loop iterates.
        while len(self.acceptlist) > 0:
            idx = random.randrange(len(self.acceptlist))
            c.originclass = self.acceptlist[idx][0]
            c.wordform = self.acceptlist[idx][1]
            c.wordforms = self.classes[c.originclass][self.acceptlist[idx][2]]
            c.stringform = c.wordforms[c.wordform]
            if self.selector is None or self.selector(c) is True:
                break
            del self.acceptlist[idx]

        if len(self.acceptlist) > 0:
            if self.postproc is not None:
                c.stringform = self.postproc(c)
        else:
            c.wordforms = None
            c.wordform = 0
            c.originclass = None
            c.stringform = None

        return c

    def __lshift__(self, other):
        """`Concatenation' operator (left-shift). Given two Replaceable
        instances A and B, the operation A << B produces a new Replaceable
        instance C with the following properties:

        * For A.cls not None and B.cls not None, C.cls will be equal to
          A.cls + B.cls. If A.cls is None, C.cls will be equal to B.cls;
          conversely if B.cls is None, C.cls will be equal to A.cls. C.cls is
          None iff A.cls is None and B.cls is None.
        * C.defaults obeys the same rules as C.cls, with the explicit
          side-note that the noncommutativity of A << B matters here: the
          defaults originating from B will only be chosen if the defaults from
          A aren't. In particular, if A.defaults contains an element with
          probability 1.0, the defaults originating from B will never be
          chosen.
        * If A.cls and A.selector are None, C.acceptlist will be equal to
          B.acceptlist. If B.cls and B.selector are None, C.acceptlist will be
          equal to A.acceptlist. Otherwise, C.acceptlist will be None.
        * If B.selector is not None, C.selector will be equal to B.selector.
          If B.selector is None, C.selector will be equal to A.selector.
          If A.selector is not None, but a catch-all C.selector is required,
          this can be achieved by setting B.selector to, for example,
            lambda x: True
          or by explicitly setting C.selector = None after calling A << B.
        * C.postproc obeys the same rules as C.selector. If a `default'
          C.postproc is desired, B.postproc can be set to
            lambda x: x
          or C.postproc can be explicitly set to None after calling A << B.

        Note that this operation is emphatically NOT commutative, i.e.
        A << B is not the same as B << A. For this reason, the left-shift
        operator was chosen as the concatenation operator, rather than the
        more obvious addition (+) operator."""

        if not isinstance(other, Replaceable):
            raise TypeError('Cannot concatenate non-Replaceable type {} to '
                'Replaceable.'.format(type(other)))

        if other.cls is None:
            cls = self.cls
        elif self.cls is None:
            cls = other.cls
        else:
            cls = self.cls + other.cls
        if other.defaults is None:
            defaults = self.defaults
        elif self.defaults is None:
            defaults = other.defaults
        else:
            defaults = self.defaults + other.defaults

        repl = Replaceable(cls, defaults,
                self.selector if other.selector is None else other.selector,
                self.postproc if other.postproc is None else other.postproc)
        if self.cls is None and self.selector is None:
            repl.acceptlist = other.acceptlist
        elif other.cls is None and other.selector is None:
            repl.acceptlist = self.acceptlist

        return repl

class IndexMap(object):
    """Class that applies a function to the replacement, given the argument
    index of that replacement. E.g. let R = ('foo', IndexMap(0, f)) be the
    replacement tuple, then this will get expanded to ('foo', f('foo'))."""
    def __init__(self, idx, f = lambda x: x):
        self.indices = (idx,) if not hasattr(idx, '__iter__') else idx
        self.func = f

    def apply(self, *data):
        return self.func(*data)

class Meme(object):
    """Meme class."""
    def __init__(self, template, *args):
        self.template = template
        self.args = args

    def __str__(self):
        """Returns a fully-filled version of the template."""
        try:
            args = [str(x() if hasattr(x, '__call__') else x) if not
                    isinstance(x, (Replaceable, IndexMap)) else
                    (x.pick() if isinstance(x, Replaceable) else x) for x in
                    self.args]

            for i in range(len(args)): # Replace IndexMaps
                if isinstance(args[i], IndexMap):
                    args[i] = args[i].apply(*[
                        args[j] for j in args[i].indices]) # ARGCEPTION

            return self.template.format(*[str(x) for x in args])
        except:
            print('>>> In template ' + self.template, file=sys.stderr)
            if args:
                print('>>> Arguments: ' + str([str(x) for x in args]),
                    file=sys.stderr)
            raise

# Praparatory block to set the scan dir and seed the PRNG if automome.py is
# called directly.
if __name__ == '__main__':
    p = os.getenv('AUTOMOME_PATH')
    if (p):
        Replaceable.scandir = p
    elif not Replaceable.scandir:
        Replaceable.scandir = os.path.abspath(os.path.dirname(sys.argv[0]))

    random.seed()

# Stock replaceables. The new syntax makes selecting the old classes more
# verbose; these stock replaceables should make the most common replaceables
# easy to select. Besides, this saves memory. :)
Rs = {
    'noun commodity':
        Replaceable(['noun'], postproc=skipart),
    'noun singular':
        Replaceable(['noun', 'noun-unique'], postproc=skipart),
    'noun plural':
        Replaceable([('noun', n_plural)]),
    'noun mass':
        Replaceable(['noun-mass'], postproc=skipart),
    'noun mass property':
        Replaceable(['noun-mass', ('adjective', a_property)],
            postproc=skipart),
    'noun singular plural':
        Replaceable([('noun', n_singular, n_plural), 'noun-unique'],
            postproc=skipart),
    'noun singular plural mass':
        Replaceable([('noun', n_singular, n_plural), 'noun-unique',
            'noun-mass'], postproc=skipart),
    'noun singular plural mass property':
        Replaceable([('noun', n_singular, n_plural), 'noun-unique',
            'noun-mass', ('adjective', a_property)], postproc=skipart),
    'noun singular mass':
        Replaceable(['noun', 'noun-unique', 'noun-mass'], postproc=skipart),
    'noun singular mass property':
        Replaceable(['noun', 'noun-unique', 'noun-mass',
            ('adjective', a_property)], postproc=skipart),
    'noun singular mass character':
        Replaceable(['noun', 'noun-unique', 'noun-mass', 'character'],
            postproc=skipart),
    'noun singular mass character skipthe':
        Replaceable(['noun', 'noun-unique', 'noun-mass', 'character'],
            postproc=curry(skipthe, skipart)),
    'noun singular mass character property':
        Replaceable(['noun', 'noun-unique', 'noun-mass', 'character',
            ('adjective', a_property)], postproc=skipart),
    'noun plural mass':
        Replaceable([('noun', n_plural), 'noun-mass'], postproc=skipart_mup),
    'noun plural mass property':
        Replaceable([('noun', n_plural), 'noun-mass',
            ('adjective', a_property)], postproc=skipart_mup),
    'noun plural mass character':
        Replaceable([('noun', n_plural), 'noun-mass', 'character'],
            postproc=skipart_mup),
    'noun plural mass character property':
        Replaceable([('noun', n_plural), 'noun-mass', 'character',
            ('adjective', a_property)], postproc=skipart_mup),
    'noun plural mass character skipthe':
        Replaceable([('noun', n_plural), 'noun-mass', 'character'],
            postproc=lambda x: skipthe_s(skipart_mup(x))),
    'noun plural mass character skipthe property':
        Replaceable([('noun', n_plural), 'noun-mass', 'character',
            ('adjective', a_property)],
            postproc=lambda x: skipthe_s(skipart_mup(x))),
    'noun singular plural mass character':
        Replaceable([('noun', n_singular, n_plural), 'noun-unique',
            'noun-mass', 'character'], postproc=skipart),
    'noun singular plural mass character property':
        Replaceable([('noun', n_singular, n_plural), 'noun-unique',
            'noun-mass', 'character', ('adjective', a_property)],
            postproc=skipart),
    'noun singular plural mass adjective':
        Replaceable([('noun', n_singular, n_plural), 'noun-unique',
            'noun-mass', 'adjective'], postproc=skipart),
    'noun singular plural mass adjective property':
        Replaceable([('noun', n_singular, n_plural), 'noun-unique',
            'noun-mass', ('adjective', a_pos, a_property)], postproc=skipart),
    'noun singular character':
        Replaceable(['noun', 'noun-unique', 'character'], postproc=skipart),
    'noun plural character':
        Replaceable([('noun', n_plural), 'character']),
    'noun commodity':
        Replaceable(['noun'], postproc=skipart),
    'noun singular article':
        Replaceable(['noun']),
    'noun singular article character':
        Replaceable(['noun', 'character']),
    'noun singular article plural mass':
        Replaceable([('noun', n_singular, n_plural), 'noun-mass'],
            postproc=skipart_mup),
    'noun singular article plural mass property':
        Replaceable([('noun', n_singular, n_plural), 'noun-mass',
            ('adjective', a_property)], postproc=skipart_mup),
    'noun singular article mass':
        Replaceable(['noun', 'noun-mass'], postproc=skipart_mup),
    'noun singular article mass property':
        Replaceable(['noun', 'noun-mass', ('adjective', a_property)],
            postproc=skipart_mup),
    'noun singular article mass character':
        Replaceable(['noun', 'noun-mass', 'character'], postproc=skipart_mup),
    'noun singular article mass character property':
        Replaceable(['noun', 'noun-mass', 'character',
            ('adjective', a_property)], postproc=skipart_mup),
    'noun interval':
        Replaceable(['noun-interval'], postproc=skipart),
    'noun interval article':
        Replaceable(['noun-interval']),
    'noun interval plural':
        Replaceable([('noun-interval', n_plural)]),

    'verb intr inf':
        Replaceable(['verb-intr']),
    'verb intr cont':
        Replaceable([('verb-intr', v_cont)]),
    'verb intr present':
        Replaceable([('verb-intr', v_present)]),
    'verb intr past':
        Replaceable([('verb-intr', v_past)]),
#    'verb intr perfect':
#        Replaceable([('verb-intr', v_perfect)]),
    'verb tr inf':
        Replaceable(['verb-tr', 'verb-tr-only']),
    'verb tr cont':
        Replaceable([('verb-tr', v_cont), ('verb-tr-only', v_cont)]),
    'verb tr present':
        Replaceable([('verb-tr', v_present), ('verb-tr-only', v_present)]),
    'verb tr past':
        Replaceable([('verb-tr', v_past), ('verb-tr-only', v_past)]),
#    'verb tr perfect':
#        Replaceable([('verb-tr', v_perfect), ('verb-tr-only', v_perfect)]),
    'verb inf':
        Replaceable(['verb-intr', 'verb-tr']),
    'verb cont':
        Replaceable([('verb-intr', v_cont), ('verb-tr', v_cont)]),
    'verb present':
        Replaceable([('verb-intr', v_present), ('verb-tr', v_present)]),
    'verb past':
        Replaceable([('verb-intr', v_past), ('verb-tr', v_past)]),
#    'verb perfect':
#        Replaceable([('verb-intr', v_perfect), ('verb-tr', v_perfect)]),

    'verb inf 1word':
        Replaceable(['verb-intr', 'verb-tr'],
            selector=lambda x: len(x.stringform.split()) == 1),

#    'adjective positive':
#        Replaceable(['adjective-positive'], postproc=skipart),
#    'adjective positive article':
#        Replaceable(['adjective-positive']),
#    'adjective positive superlative':
#        Replaceable([('adjective-positive', a_super)]),
#    'adjective neutral':
#        Replaceable(['adjective-neutral'], postproc=skipart),
#    'adjective neutral article':
#        Replaceable(['adjective-neutral']),
#    'adjective neutral superlative':
#        Replaceable([('adjective-neutral', a_super)]),
    'adjective negative':
        Replaceable(['adjective-negative'], postproc=skipart),
#    'adjective negative article':
#        Replaceable(['adjective-negative']),
#    'adjective negative superlative':
#        Replaceable([('adjective-negative', a_super)]),
    'adjective':
        Replaceable(['adjective'], postproc=skipart),
    'adjective article':
        Replaceable(['adjective']),
    'adjective superlative':
        Replaceable([('adjective', a_super)]),

    'adverb':
        Replaceable(['adverb']),

    'character':
        Replaceable(['character']),
    'character skipthe':
        Replaceable(['character'], postproc=skipthe),

    'place':
        Replaceable(['place']),
    'place prep':
        Replaceable(['place'], postproc=lambda x: x.wordforms[1] + ' ' +
            x.wordforms[0]),

    'name-syllable':
        Replaceable(['ed-name-syllable'])
}

# Easy replacement constructor that uses varargs for the class list and kwargs
# for other other arguments:
R = lambda *x, **y: Replaceable(x, **y)
# Easy defaults-only replacement constructor:
C = lambda *x, **y: Replaceable(cls=None, defaults=x, **y)

def cavememe():
    """Returns the main part of the Colossal Cave Adventure meme."""
    place = R('place', postproc=lambda x: skipthe(x) + ' OF').pick()
    adj = R('adjective', defaults=[('RIVERISH', 0.5)],
            postproc=skipart).pick()
    tup = random.choice(tuple(itertools.permutations((place, adj,
        random.choice(('TWISTING', 'TWISTY'))))))
    prep = 'A'
    if place.originclass == 'place':
        if place.wordforms[2] == 'SOME':
            prep = 'SOME'
        elif tup[0] == place:
            prep = place.wordforms[2]
    elif tup[0] == adj and adj.originclass == 'adjective':
        prep = adj.wordforms[0].split()[0]

    return ' '.join([place.wordforms[1], prep] + [str(x) for x in tup])

# Lambda to generate a random name using Eternal Density's syllable list. s is
# the number of syllables; default is a (string, probability) tuple, like
# entries in the defaults parameter in a Replaceable, but only one default can
# be specified here.
edname = lambda s = 1, default = None: lambda: \
        ''.join([str(Rs['name-syllable'].pick()) for i in range(s)]) \
            if default is None or random.random() < (1 - default[1]) \
            else default[0]

# For the origins of these memes, see mrob27's list at
#   http://mrob.com/time/automome/ott-snowclones.txt
memes_automeme_molpish = [
    Meme('ALL YOUR {} ARE BELONG TO {}',
        Rs['noun singular'],
        Rs['noun singular plural mass character']),
    Meme('ATTACK ITS WEAK POINT FOR {} {}',
        Rs['adjective'],
        Rs['noun singular plural mass property']),
    Meme('BECAUSE YOU ARE _{} {}_',
        Rs['adjective'] << C(('HEADCRAB', 0.33)),
        Rs['noun singular mass']),
    Meme('BECOME AN {}',
        Rs['noun singular plural mass'] << R(selector=lambda x: \
            x.originclass != 'noun' or x.wordform == n_plural \
            or x.stringform[:2] == 'A ')),
    Meme('BETTER {} MY OWN {}',
        Rs['verb tr inf'],
        Rs['noun singular mass property']),
    Meme('BLESS YOU, {}. BL{}.',
        Rs['noun plural mass character skipthe property'],
        IndexMap(0, skipuntilvowel)),
    Meme('CEILING {} IS WATCHING YOU {}',
        Rs['noun singular character'],
        Rs['verb inf']),
    Meme('CH*RPING {}, HOW DO THEY WORK?',
        Rs['noun plural']),
    Meme('DAMNIT {}, I\'M {}, NOT {}',
        Rs['character skipthe'] << C(('JIM', 0.5)),
        Rs['noun singular article'],
        Rs['noun singular article']),
    Meme('DISREGARD THAT, I {} {}',
        Rs['verb tr inf'],
        Rs['noun plural mass property']),
    Meme('DR. {}, OR HOW I LEARNED TO STOP {} AND LOVE THE {}',
        Rs['noun singular mass character'],
        Rs['verb intr cont'] << C(('WORRYING', 0.1)),
        Rs['noun singular']),
    Meme('GREAT LORD RANDALL IT\'S {} GET IN THE {}',
        Rs['noun singular mass character property'] << R(postproc=lambda x: \
            'THE ' + skipword(x.stringform) \
                if x.originclass == 'noun-unique' else skipart_mup(x)),
        Rs['noun singular mass'] << C(('CAR', 0.4), ('RAFTCASTLE', 0.4))),
    Meme('HELLO. MY NAME IS {}. YOU {} MY {}. PREPARE TO {}.',
        Rs['character'],
        Rs['verb tr past'],
        Rs['noun singular plural mass property'],
        Rs['verb intr inf']),
    Meme('HELLO? YES, THIS IS {}',
        Rs['noun singular mass character property']),
    Meme('HOW DO I {} {}?',
        Rs['verb tr past'],
        Rs['noun singular plural mass character property']),
    Meme('HOW IS {} FORMED? HOW {} GET {}?',
        Rs['noun singular mass property'],
        Rs['noun singular mass character property'],
        Rs['adjective'] << C(('PRAGNENT', 0.33))),
    Meme('I ACCIDENTALLY {}, IS THIS {}?',
        Rs['noun singular article'],
        C(('BAD', 0.5), ('DANGEROUS', 0.5), ('ENDISH', 1.0))),
    Meme('I AM _{0} {1}_, THE {0} {1} EVER!',
        R(('adjective', a_super),
            defaults=[('BEST', 0.4), ('MOLPIEST', 0.67)], selector=lambda x: \
                x.stringform.endswith('EST') \
                and not x.stringform.startswith('MOST')),
        Rs['noun singular mass property']),
    Meme('I AM {}, HEAR ME {}',
        Rs['character'],
        Rs['verb intr inf'] << C(('ROAR', 0.5))),
    Meme('I DON\'T ALWAYS {}{}, BUT WHEN I DO, I {} {}',
        R('verb-intr', 'verb-tr', 'verb-tr-only'),
        IndexMap(0, lambda x: '' if x.originclass == 'verb-intr' else \
            (' ' + Rs['noun plural mass property'].pick().stringform)),
        Rs['verb tr inf'] << C(('PREFER', 0.2)),
        Rs['noun plural mass property']),
    Meme('I, FOR ONE, WELCOME OUR NEW {} OVERLORDS',
        Rs['noun singular']),
    Meme('I HAS {}. YOU CAN HAS MY {}.',
        Rs['noun singular article'],
        IndexMap(0, skipword)),
    Meme('I HAVE REPORTED YOU TO {} FOR {} {} AS IT IS A CRIME',
        Rs['character'],
        Rs['verb tr cont'] << C(('STEALING', 0.75)),
        Rs['noun plural mass']),
    Meme('I LIKE MY {} LIKE I LIKE MY {}: COVERED IN {}',
        Rs['noun plural mass property'] << C(('WOMEN', 0.25), ('MEN', 0.33)),
        Rs['noun plural mass property'],
        Rs['noun plural mass property']),
    Meme('I MADE YOU {} BUT I {} IT',
        Rs['noun singular article'],
        Rs['verb tr past']),
    Meme('IMMA CHARGIN MAH {}',
        Rs['noun singular plural mass property']),
    Meme('I\'M IN UR {}, {} UR {}',
        Rs['noun singular mass property'],
        Rs['verb tr cont'],
        Rs['noun plural mass property']),
    Meme('IN B4 {} {}',
        Rs['adjective'] << C(('503', 0.2), ('408', 0.25)),
        Rs['noun singular plural mass property']),
    Meme('I SEE WHAT {} {} THERE',
        Rs['character'] << C(('YOU', 0.5)),
        Rs['verb tr past']),
    Meme('IS IT CAN BE {} TIEM NAO PLEES?',
        R(('noun', n_singular, n_plural), 'noun-unique',
            selector=lambda x: x.wordform == n_singular \
                or x.stringform.endswith('S'),
            postproc=lambda x: (x.stringform[:-1] + 'Z') \
                if x.wordform == n_plural else skipword(x))),
    Meme('I THINK {} IS A PRETTY {} {}. '
            'XEH {} {} AND DOESN\'T AFRAID OF ANYTHING',
        Rs['character'],
        Rs['adjective'] << C(('TREEISH', 0.1), ('MOLPISH', 0.11)),
        Rs['noun singular'] << C(('XUY', 0.1)),
        Rs['verb tr present'],
        Rs['noun plural mass character property']),
    Meme('I WILL NOT {} THIS {} -- IT IS {}',
        Rs['verb tr inf'],
        Rs['noun singular mass property'],
        R('noun', 'noun-unique', 'noun-mass',
            ('adjective', a_pos, a_property), 'character',
            postproc=lambda x: skipword(x) if x.originclass == \
                'adjective' else (('THE ' + skipword(x)) if
                    x.originclass == 'noun-unique' else skipart_mup(x)))),
    Meme('I {} _ALL_ THE {}',
        Rs['verb tr past'],
        Rs['noun plural mass property']),
    Meme('I {}, THEREFORE I {}',
        R('verb-intr', defaults=[('THINK', 0.5)]),
        Rs['verb intr inf']),
    Meme('MAXIMUM {} YIELDS MAXIMUM {}',
        Rs['noun plural mass property'],
        Rs['noun plural mass property']),
    Meme('MY {} IS FULL OF {}',
        Rs['noun singular mass'],
        Rs['noun plural mass property']),
    Meme('MY {} JUST {} ME; M*STARD WAS SO {}',
        Rs['noun singular mass'],
        Rs['verb tr past'],
        Rs['adjective'] << C(('CASH', 0.5))),
    Meme('MY {}, LET ME SHOW YOU IT',
        Rs['noun singular mass']),
    Meme('NOOOO THEY BE {} MY {}',
        Rs['verb tr cont'],
        Rs['noun singular plural mass property']),
    Meme('NOT SURE IF {} OR JUST {}',
        Rs['verb intr cont'],
        R(('verb-intr', v_cont), 'adjective', postproc=skipart)),
    Meme('NO, {}, YOU ARE THE {}',
        Rs['noun singular mass character property'],
        Rs['noun singular mass property']),
    Meme('OH GLR HOW DID {} GET HERE I AM NOT GOOD WITH {}',
        R(('noun', n_plural), 'noun-mass', 'character',
            ('adjective', a_property), defaults=[('THIS', 0.5)],
            postproc=skipart_mup),
        Rs['noun plural mass']),
    Meme('OH HAI, I {} YOUR {}',
        Rs['verb tr past'],
        Rs['noun singular plural mass property']),
    Meme('PROTIP: TO {} THE {}, {} IT UNTIL IT {}',
        Rs['verb tr inf'] << C(('DEFEAT', 0.25)),
        Rs['noun singular mass'],
        Rs['verb tr inf'],
        Rs['verb intr present']),
    Meme('PUSH {}, RECEIVE {}',
        C(('BUTAN', 0.5), ('BUTTON', 1.0)),
        Rs['noun singular plural mass']),
    Meme('SCREAMING {}: {} DOESN\'T LIKE IT',
        Rs['noun singular character'],
        Rs['character']),
    Meme('SO I HERD U {} {}',
        Rs['verb tr inf'] << C(('LIEK', 0.75)),
        Rs['noun plural mass property']),
    Meme('THANKS, {}. TH{}.',
        Rs['noun plural mass character skipthe property'],
        IndexMap(0, skipuntilvowel)),
    Meme('THANK YOU {}, BUT OUR {} IS IN ANOTHER {}',
        Rs['character'],
        Rs['noun singular mass'],
        Rs['noun singular']),
    Meme('THAT\'S NOT {0}; _THIS_ IS {0}',
        Rs['noun singular article']),
    Meme('THERE IS {} IN MY {}, YOUR ARGUMENT IS INVALID',
        Rs['noun singular article'],
        Rs['noun singular mass'] << C(('BEARD', 0.33))),
    Meme('THEY SEE ME {}, THEY {}',
        R(('verb-intr', v_cont),
            selector=lambda x: x.stringform.endswith('ING'),
            postproc=lambda x: x.stringform[:-1] + '\''),
        R(('verb-intr', v_cont), defaults=[('HATIN\'', 0.33)],
            selector=lambda x: x.stringform.endswith('ING'),
            postproc=lambda x: x.stringform[:-1] + '\'')),
    Meme('THE {} IS A LIE',
        Rs['noun singular']),
    Meme('THIS IS YOUR {} ON {}',
        Rs['noun singular'] << C(('BRAIN', 0.5)),
        Rs['noun plural mass property']),
    Meme('THIS IS {}. I CAN TELL BY THE {}, '
            'AND FROM HAVING SEEN A LOT OF {} IN MY DIP',
        R(('verb-tr', v_perfect), 'adjective',
            defaults=[('PHOTOSHOPPED', 0.12), ('OTTIFIED', 0.14),
                ('SHOOPED', 0.12)], postproc=skipart),
        Rs['noun plural mass property'],
        Rs['noun plural mass property']),
    Meme('THIS IS {} {}, I MUST {} IT',
        Rs['adjective'],
        Rs['noun singular'],
        Rs['verb tr inf']),
    Meme('THIS IS {} {}. IT IS MADE OF {} AND {}.',
        Rs['adjective article'],
        Rs['noun commodity'],
        Rs['noun singular plural mass adjective property'],
        Rs['noun singular plural mass adjective property']),
    Meme('THIS {} IS NOW ABOUT {} {}',
        R('noun', 'noun-unique', defaults=[('NEEDLE-PULLED THING', 0.5)],
            postproc=skipart),
        Rs['adjective'],
        Rs['noun plural mass character skipthe property']),
    Meme('VERY IMPORTANT FOR {} TO {} XIM {}',
        Rs['character'],
        R('verb-tr', 'verb-tr-only', defaults=[('DO', 0.25)]),
        R('noun', 'noun-unique', 'noun-mass', ('adjective', a_property),
            defaults=[('JOB', 0.5)], postproc=skipart)),
    Meme('WE\'VE SECRETLY REPLACED {}\'S {} WITH {}. '
            'LET\'S SEE IF XE NOTICES...',
        Rs['character'],
        Rs['noun singular mass'],
        Rs['noun singular article']),
    Meme('WHAT _ARE_ {}? WE JUST DON\'T KNOW.',
        Rs['noun plural']),
    Meme('WHATEVER {} YOUR {}',
        Rs['verb tr present'],
        R(('noun', n_singular, n_plural), 'noun-unique', 'noun-mass',
            defaults=[('RAFTCASTLE', 0.67), ('SHIPCASTLE', 0.33)],
            postproc=skipart)),
    Meme('WHAT THE CH*RP? YOU ARE {}. AND {}',
        Rs['noun singular mass character'],
        Rs['adjective']),
    Meme('YOU HAVE {} MOST OF A RIVERISH {}. '
            'PLEASE PICK YOUR {} WITH GREATER CARE.',
        Rs['verb tr past'],
        Rs['noun commodity'],
        Rs['noun plural mass']),
    Meme('YOU WOULDN\'T {} {}',
        R('verb-tr', 'verb-tr-only', defaults=[('DOWNLOAD', 0.5)]),
        Rs['noun singular article']),
    Meme('YO WAWLPY, I HEARD YOU LIKE {0} SO WE PUT {1} IN YOUR {2} '
            'SO YOU CAN {3} WHILE YOU {3}',
        IndexMap(1, lambda x: x.wordforms[n_plural] \
            if x.originclass == 'noun' else skipart_mup(x)),
        R(('noun', n_singular, n_plural), 'noun-mass',
            ('adjective', a_property), postproc=lambda x: \
            ('SOME ' + skipart_mup(x)) if x.originclass in \
            ('noun-mass', 'adjective') or x.wordform == n_plural \
            else x.stringform),
        IndexMap(1, lambda x: x.stringform if x.originclass == 'noun' \
            and x.wordform == n_plural \
            else skipword(x.wordforms[x.wordform])),
        Rs['verb intr inf']),
    Meme('YO {}, I\'M REALLY HAPPY FOR YOU, IMMA LET YOU FINISH, '
            'BUT {} HAD ONE OF THE {} {} OF ALL TIME',
        Rs['character skipthe'],
        Rs['character'],
        C(('MOST TREEISH', 0.33), ('MOLPIEST', 0.5), ('MOST WOWTERFUL', 1.0)),
        Rs['noun plural']),
    Meme('{} _ALL_ THE {}',
        Rs['verb tr inf'],
        Rs['noun plural mass property']),
    Meme('{} DON\'T KNOW ABOUT MY {}',
        Rs['noun plural'],
        Rs['noun singular plural mass']),
    Meme('{}? IN _MY_ {}?',
        Rs['noun plural mass property'],
        Rs['noun singular mass']),
    Meme('"{}!" I SCREAMED AS {} {}',
        R('interjection', defaults=[('THIS IS NOT ACCEPTABLE', 0.8)]),
        Rs['character'],
        Rs['verb intr past']),
    Meme('{} IS SERIOUS {}',
        Rs['noun singular character'],
        Rs['noun singular']),
    Meme('{} IS THE {} OF THE {}',
        Rs['noun singular character'],
        Rs['noun singular mass property'],
        Rs['noun plural mass']),
    Meme('{} IS {} TOO',
        Rs['noun singular article'],
        Rs['adjective']),
    Meme('{} LOEV {}',
        R('character', defaults=[('LOLI', 0.5)]),
        Rs['noun singular plural mass character property']),
    Meme('{0}MEOWLPY IS {1}',
        IndexMap(1, curry(lambda x: x[:-5] if x.endswith('MOLPY') and \
            x != 'MOLPY' else x, skipword)),
        Rs['noun singular article']),
    Meme('{}\'S CLOSED DUE TO {}',
        R('noun', 'noun-unique', 'noun-mass', 'place',
            defaults=[('POOL', 0.5)], postproc=lambda x: skipthe(x) if \
                x.originclass == 'place' else skipart(x)),
        Rs['noun plural mass character property']),
    Meme('{} OR GTCO',
        Rs['noun plural mass property']),
    Meme('{} THE {}!',
        Rs['verb tr inf'],
        Rs['noun plural mass property']),
    Meme('{} THE {}... WITH YOUR {}',
        Rs['verb tr inf'],
        Rs['noun plural mass property'],
        Rs['noun singular plural mass']),
    Meme('{} {} ARE _SUPERIOR_',
        Rs['adjective'],
        Rs['noun plural']),
    Meme('{} {}, {} {} _YOU!_',
        IndexMap(1, lambda x: x.wordforms[1] if x.originclass == 'place' \
            else 'IN'),
        R('place', defaults=[('SOVIET RUSSIA', 0.5)],
            postproc=lambda x: 'THE SOVIET ' + x.stringform[4:] \
                if x.stringform.startswith('THE ') else 'SOVIET ' + \
                x.stringform),
        Rs['noun singular mass character property'],
        Rs['verb tr present'])
]

memes_other_molpish = [
    Meme('ACHIEVEMENT UNLOCKED: {} {}',
        Rs['verb tr inf'] << C(('GO TO', 0.05), ('CONQUER', .053),
            ('EXPLORE', 0.056)),
        IndexMap(0, lambda x: R('noun', 'noun-mass', 'noun-unique',
                'character', ('adjective', a_property),
                postproc=lambda y: skipword(y) if y.originclass in \
                    ('adjective', 'noun-mass') \
                    else ('THE ' + skipword(y) \
                        if y.originclass == 'noun-unique' \
                        else y.stringform)).pick() \
            if x.originclass != 'x-defaults' else Rs['place'].pick())),
    Meme('AH! {} YOUR {} BUT INEVITABLE {}!',
        R('verb-tr', defaults=[('CURSE', 0.8)]),
        Rs['adjective'],
        Rs['noun singular plural mass property']),
    Meme('ALSO, I CAN {} YOU WITH MY {}.',
        Rs['verb tr inf'],
        Rs['noun singular']),
    Meme('AND WHY DO YOU WORRY ABOUT {}? CONSIDER THE {} OF {}, '
            'HOW THEY {}: THEY NEITHER {} NOR {}.',
        Rs['noun plural'],
        Rs['noun plural'],
        Rs['place'],
        Rs['verb inf'],
        Rs['verb inf'],
        Rs['verb inf']), # Credit: mrob27
    Meme('A WILD {} {}',
        Rs['noun singular'],
        C(('APPEARS', 0.5), ('APPEARED', 1.0))), # Credit: mrob27
    Meme('BE VEWY VEWY {}, I\'M {} {}',
        R('adjective', defaults=[('QUIET', 0.8)],
            postproc=curry(fudd, skipart)),
        R(('verb-tr', v_cont), ('verb-tr-only', v_cont),
            defaults=[('HUNTING', 0.5)], postproc=fudd),
        R(('noun', n_plural), postproc=fudd)),
    Meme('BRACE {}, {} IS COMING',
        C(('YOURSELF', 0.5), ('YOURSELVES', 1.0)),
        Rs['noun singular mass property']), # Credit: mrob27
    Meme('CONSIDER THE {} OF {}',
        Rs['noun plural mass property'],
        Rs['place']), # Credit: mrob27
    Meme('COULD {} _BE_ ANY MORE {}?',
        Rs['noun plural mass character property'],
        Rs['adjective']),
    Meme('CROUCHING {}, HIDDEN {}',
        Rs['noun singular'],
        Rs['noun singular']),
    Meme('DIAL {} FOR {}',
        IndexMap(1, lambda x: x.stringform[0]),
        Rs['noun singular plural mass character property']),
    Meme('DON\'T {} ME BECAUSE I\'M {}',
        R('verb-tr', 'verb-tr-only', defaults=[('HATE', 0.3)]),
        R('adjective', defaults=[('FLUTTERBEEWINGISH', 0.2),
            ('WINGISH', 0.25)], postproc=skipart)), # Credit: mrob27
    Meme('ET TU, {}?',
        Rs['character skipthe']),
    Meme('EVERY TIME YOU {}, {} {} {}. PLEASE, THINK OF THE {}.',
        Rs['verb intr inf'],
        R('character', defaults=[('RANDALL', 0.3), ('THE GLR', 0.43)]),
        Rs['verb tr present'],
        Rs['noun singular article'],
        IndexMap(3, lambda x: x.wordforms[n_plural])), # Credit: mrob27
    Meme('EXCUSE ME {}, DO YOU HAVE A MOMENT TO TALK ABOUT {}?',
        C(('SIR', 0.5), ('MA\'AM', 1.0)),
        R(('noun', n_plural), 'character', 'noun-unique', 'noun-mass',
            ('adjective', a_property), postproc=uniquethe)),
    Meme('GOT {}?',
        R(('noun', n_plural), 'noun-mass', 'noun-mass',
            ('adjective', a_property), postproc=skipart)),
        # Credit: mrob27
    Meme('HAVE {}, WILL {}',
        Rs['noun singular mass property'],
        R(('verb-intr', v_inf), ('verb-tr', v_inf), defaults=[
            ('TIME-TRAVEL', 0.1), ('TRAVEL', 0.4)])), # Credit: mrob27
    Meme('HELLO, I\'M {}. BASICALLY, {}!',
        Rs['character'],
        R('verb-intr', defaults=[('RUN', 0.67)])),
    Meme('♫ HEY, I JUST {} YOU, AND THIS IS CRAZY / '
        'BUT HERE\'S MY {}, SO {} ME MAYBE ♫',
        Rs['verb tr past'],
        Rs['noun singular mass'],
        Rs['verb tr inf']),
    Meme('I CH*RPING LOVE {}',
        Rs['noun plural mass property']),
    Meme('I DIDN\'T CHOOSE THE {0} LIFE; THE {0} LIFE CHOSE ME',
        Rs['noun singular mass']),
    Meme('I FIND YOUR LACK OF {} DISTURBING',
        Rs['noun plural mass property']),
    Meme('IF YOUR HAND TOUCHES {}, I SWEAR BY MY {} {} {}, I _WILL_ {} YOU.',
        Rs['noun mass'],
        C(('PRETTY', 0.6), ('WINGISH', 0.8), ('FLUTTERBEEWINGISH', 1.0)),
        R('adjective', defaults=[('FLORAL', 0.8)], postproc=skipart),
        Rs['noun singular'],
        R('verb-tr', defaults=[('**D', 0.25)])),
    Meme('IF {0} ARE THE {1} OF THE {2}, AND {1} ARE THE {3} OF THE {4}, '
        'WHY ARE WE STILL {5} {0}?',
        Rs['noun plural'],
        Rs['noun plural'],
        Rs['noun singular plural mass'],
        Rs['noun plural'],
        Rs['noun singular plural mass'],
        Rs['verb tr cont']), # Credit: mrob27
    Meme('IF {} {} WRONG, I DON\'T WANT TO BE RIGHT',
        R(('verb-intr', v_cont), ('verb-tr', v_cont),
            ('verb-intr', v_cont), ('verb-tr', v_cont),
            ('noun', n_plural), 'noun-unique', 'noun-mass', 'character',
            ('adjective', a_property), postproc=uniquethe),
        IndexMap(0, lambda x: 'ARE' if x.originclass == 'noun' else 'IS')),
        # Credit: mrob27
    Meme('IF {} {} IS {}, {} {}.',
        Rs['verb tr cont'],
        R(('noun', n_plural), 'place', defaults=[('IN YOUR PANTS', 0.3)],
            postproc=lambda x: ('IN ' if x.originclass == 'place' \
                else 'YOUR ') + x.stringform),
        Rs['adjective'],
        C(('THEN I\'M', 0.5), ('CONSIDER ME', 1.0)),
        R(('verb-tr', v_perfect), 'adjective', 'character',
            postproc=skipart)), # Credit: mrob27
    Meme('I HAVEN\'T GOT _ANYTHING_ AGAINST THE {}! CONSIDER THE {}.',
        Rs['noun plural'],
        Rs['noun plural']), # Credit: mrob27
    Meme('I\'M GOING TO {0}{1} LIKE I\'VE NEVER {2}{1} BEFORE.',
        Rs['verb inf'],
        IndexMap(0, lambda x: '' if x.originclass == 'verb-intr' \
            else (' ' + Rs['noun plural'].pick().stringform)),
        IndexMap(0, lambda x: x.wordforms[v_perfect])),
    Meme('I\'M GOING TO {0} {1} {2}, AND BY \'{0}\' I MEAN \'{3}\', '
            'AND BY \'{1} {2}\' I MEAN \'{4}\'.',
        Rs['verb tr inf'],
        C(('MY', 0.5), ('THIS', 1.0)),
        Rs['noun singular'],
        Rs['verb tr inf'],
        Rs['noun plural mass property']), # Credit: mrob27
    Meme('I\'M NOT {} BUT I {} ONE {}',
        Rs['noun singular article'],
        R('verb-tr', 'verb-tr-only', defaults=[('PLAY', 0.33)]),
        Rs['place prep']), # Credit: mrob27
    Meme('I\'M SORRY {}, I\'M AFRAID I CAN\'T {} THAT.',
        Rs['character'],
        R('verb-tr', 'verb-tr-only', defaults=[('DO', 0.15)])),
    Meme('I\'M SUCH A LEFT {} WHEN IT COMES TO {}',
        Rs['noun singular'] << R(defaults=[('RAPTORSHARK', 0.5)],
            selector=lambda x: len(str(x).split()) == 2),
        Rs['verb cont'] << R(selector=lambda x: len(str(x).split()) == 1)),
    Meme('I\'M WORKING TO {} {} {} THAT WILL ETERNALLY {} {}{} WHO {} TO {}'
            ' THE {}\'S {} PEOPLE.',
        R('verb-tr', defaults=[('CREATE', 0.25), ('BRING ABOUT', 0.33)]),
        Rs['adjective article'],
        Rs['noun singular mass'],
        Rs['verb tr inf'],
        C(('EVERY', 0.5), ('ANY', 1.0)),
        C(('ONE', 0.5), ('MOLPY', 1.0)),
        C(('FAILS', 0.5), ('FAILED', 1.0)),
        Rs['verb tr inf'],
        R('character', selector=lambda x: len(x.stringform.split()) == 1),
        Rs['noun singular mass']), # Credit: mrob27
    Meme('INSTANT {}, JUST ADD {}',
        Rs['noun singular plural mass property'],
        Rs['noun plural mass property']),
    Meme('IN YOUR MIND, REPLACE {} WITH {}.',
        Rs['noun singular mass'],
        Rs['noun singular mass']), # Credit: Neil_Boekend via mrob27
    Meme('I REJECT YOUR {} AND SUBSTITUTE MY OWN',
        Rs['noun singular plural mass property']),
    Meme('IS THAT {} IN {} {} OR {} JUST HAPPY TO {} ME?',
        Rs['noun singular article'],
        Rs['character'] << R(defaults=[('YOUR', 0.75)], postproc=lambda x: \
            x.stringform if x.originclass == 'x-defaults' \
            else x.stringform + '\'S'),
        Rs['noun singular mass'] << C(('PANTS', 0.15), ('POCKET', 0.18)),
        IndexMap(1, lambda x: 'ARE YOU' if x.originclass == 'x-defaults' \
                else 'IS XE'),
        Rs['verb tr inf'] << C(('SEE', 0.25))),
    Meme('IT\'S {} ALL THE WAY DOWN{}',
        Rs['noun plural mass property'],
        IndexMap(0, lambda x:
            (', EXCEPT THAT EVERY 13TH AND 14TH ONE IS {}'.format(str(
                Rs['noun singular article'].pick()))) \
            if x.originclass == 'noun' and x.wordform == n_plural \
            and random.choice((True, False)) else '')), # Credit: mrob27
    Meme('IT\'S-A-ME, {}!',
        Rs['character']),
    Meme('IT\'S {} GONE WILD{}!',
        Rs['noun plural'],
        C(('', 0.3), (' II', 0.3), (' III', 0.3), (' 4', 0.3), (' 5', 0.3),
            (' 6', 0.4), (' 7', 1.0))), # Credit: mrob27
    Meme('IT\'S {}. IT\'S {}. IT\'S _SUPER{}_!',
        Rs['noun singular article'],
        Rs['noun singular article'],
        R('noun', 'noun-unique', 'noun-mass',
            selector=lambda x: len(x.stringform.split()) == (2 if \
                x.originclass == 'noun' else 1), postproc=skipart)),
    Meme('I USED TO BE {} LIKE YOU, THEN I TOOK {} IN THE KNEE',
        Rs['noun singular article'],
        Rs['noun singular article']),
    Meme('I\'VE GOT A JAR OF {0}! I\'VE GOT A JAR OF {0}! '
            'AND GUESS WHAT\'S INSIDE IT!',
        Rs['noun plural mass']),
    Meme('"I WANT THE {0}!"/"YOU CAN\'T HANDLE THE {0}!"',
        Rs['noun singular mass property']),
    Meme('I WEAR {} NOW. {} ARE {}.',
        Rs['noun singular article'],
        IndexMap(0, lambda x: x.wordforms[n_plural]),
        C(('COOL', 0.5), ('TREEISH', 0.5), ('MOLPISH', 1.0))),
    Meme('I WENT {} {}{} AND {} {}BROKE OUT.',
        C(('TO SEE', 0.5), ('TO', 1.0)),
        Rs['noun singular article'] << C(('', 0.5)),
        IndexMap(1, lambda x: '-' + str(Rs['verb tr cont'].pick()) \
            if x.originclass != 'x-defaults' \
            else R('place', postproc=lambda x: \
                x.wordforms[0] if x.wordforms[0].startswith('THE ') \
                else x.wordforms[2] + ' ' + x.wordforms[0]).pick()),
        Rs['noun singular article mass property'],
        C(('ALMOST ', 0.5), ('', 1.0))), # Credit: mrob27
    Meme('I {} {} AND ALL I GOT WAS THIS LOUSY T-SHIRT',
        IndexMap(1, lambda x: 'WENT TO' if x.originclass == 'place' \
            else Rs['verb tr past'].pick()),
        R('noun', 'place')),
    Meme('KEEP CALM AND {} ON',
        R('verb-intr', defaults=[('MOLPY', 0.1)],
            selector=lambda x: len(x.stringform.split()) == 1)),
        # Credit: mrob27
    Meme('LISTEN, DON\'T MENTION THE {}!',
        R(('noun', n_singular, n_plural), 'noun-unique', 'noun-mass',
            ('adjective', a_property), postproc=skipart)),
    Meme('LOOK ON MY {}, YE {}, AND {}!',
        Rs['noun plural mass'],
        R(('verb-intr', v_cont), 'adjective',
            selector=lambda x: x.originclass == 'adjective' or \
                (x.stringform.endswith('ING') and \
                    len(x.stringform.split()) == 1), postproc=skipart),
        R('verb-intr', defaults=[('DESPAIR', 0.25)])),
    Meme('MY NAME IS {}, {} OF {}: LOOK ON MY {}, YE {}, AND {}!',
        Rs[ 'character'],
        Rs['noun commodity'],
        IndexMap(1, lambda x: x.wordforms[n_plural]),
        Rs['noun plural mass'],
        R(('verb-intr', v_cont), 'adjective',
            selector=lambda x: x.originclass == 'adjective' or \
                (x.stringform.endswith('ING') and \
                    len(x.stringform.split()) == 1), postproc=skipart),
                R('verb-intr', defaults=[('DESPAIR', 0.25)])),
            # Credit for this form: mrob27
    Meme('MY {} BRINGS ALL THE {} TO {}',
        Rs['noun singular'],
        Rs['noun plural'],
        R('place', defaults=[('THE YARD', 0.5)])),
    Meme('MY {} IS PROBLEMATIC',
        Rs['noun singular']),
    Meme('MY {} {} OVER',
        Rs['noun singular mass'],
        R(('verb-intr', v_present), ('verb-tr', v_present), selector=lambda x:
            x.stringform.endswith('S'), postproc=lambda x: re.sub(
                r'(.)S$', r'\1\1ETH', re.sub(r'TS$', 'TETH', re.sub(
                r'([AIOU])S$', r'\1ETH', re.sub(r'ES$', 'ETH', x.stringform)
        ))))), # Credit: mrob27
    Meme('_MY {} {}: {} IS {}_',
        C(('LITTLE', 0.75), ('RIVERISH', 1.0)),
        Rs['noun singular'],
        R(('adjective-positive', a_property), defaults=[('FRIENDSHIP', 0.25)],
            postproc=skipart),
        R('adjective-positive', defaults=[('MAGIC', 0.5)], postproc=skipart)),
    Meme('NEVER SEND TO KNOW FOR WHOM THE {0} {1}; IT {1} FOR THEE',
        Rs['noun singular'],
        Rs['verb present']), # Credit: mrob27
    Meme('NOW I AM BECOME {}, THE {} OF {}',
        Rs['noun singular mass character property'],
        R(('verb-tr', v_cont), selector=lambda x: \
                x.stringform.endswith('ING'), postproc=erify),
        Rs['noun plural mass property']),
    Meme('NOW IF YOU\'LL EXCUSE ME, I HAVE {} TO {}',
        Rs['noun singular article plural mass property'],
        Rs['verb tr inf']),
    Meme('NOW IS THE TIME FOR ALL {} {} TO COME TO THE AID OF {}',
        C(('TREEISH', 0.33), ('MOLPISH', 0.5), ('GOOD', 1.0)),
        Rs['noun plural'],
        R('noun', 'noun-unique', 'place', postproc=lambda x: \
            ('THE ' + skipart(x)) if x.originclass.startswith('noun-') \
            else x.stringform)), # Credit: mrob27
    Meme('NO {} WERE {} IN THE MAKING OF THIS {}.',
        R(('noun', n_plural), defaults=[('MOLPIES', 0.1)]),
        R(('verb-tr', v_perfect), defaults=[('HARMED', 0.5)]),
        R('noun', 'noun-mass', defaults=[('MEME', 0.3)], postproc=skipart)),
        # Credit: azule via mrob27
    Meme('ONE DOES NOT SIMPLY {} INTO {}',
        Rs['verb intr inf'],
        R('place', defaults=[('MORDOR', 0.3)])), # Credit: mrob27
    Meme('ONE {} DOES NOT {} MAKE',
        Rs['noun commodity'],
        Rs['noun singular article']),
    Meme('ONE {0} TO {1} THEM ALL, ONE {0} TO FIND THEM.'
            ' ONE {0} TO {2} THEM ALL AND IN {3} BIND THEM.',
        Rs['noun singular'],
        R('verb-tr', 'verb-tr-only', defaults=[('RULE', 0.75)]),
        R('verb-tr', 'verb-tr-only', defaults=[('BRING', 0.75)]),
        R('place', selector=lambda x: x.wordforms[1] == 'IN')),
    Meme('ONLY TWO THINGS COME FROM {}: {} AND {}',
        Rs['place'],
        Rs['noun plural mass character property'],
        Rs['noun plural mass character property']),
    Meme('O {0}, {0}! WHEREFORE ART THOU {0}?',
        Rs['character skipthe']),
    Meme('PH\'NGLUI MGLW\'NAFH {} {} WGAH\'NAGL FHTAGN',
        Rs['character skipthe'],
        R('place', postproc=skipthe)),
    Meme('PLEASE READ: A PERSONAL APPEAL FROM {} FOUNDER {}.',
        Rs['noun singular'],
        Rs['character']), # Credit: mrob27
    Meme('PUT {} ON {}',
        Rs['noun singular mass'],
        R('noun', 'noun-unique', 'noun-mass', defaults=[('HEAD', 0.5)],
            postproc=skipart)),
    Meme('REMAIN {} AND {} {}',
        R('adjective', defaults=[('CALM', 0.2)], postproc=skipart),
        Rs['verb tr inf'],
        Rs['noun plural mass']), # Credit: mrob27
    Meme('SAVE {}, {} {}',
        Rs['noun singular article'],
        R('verb-tr', defaults=[('EAT', 0.12), ('RIDE', 0.15)]),
        Rs['noun singular article']), # Credit: azule via mrob27
    Meme('SEE, {} AND {}, I GOT NO PROBLEM WITH, LONG AS XE DOES IT {}-LIKE',
        Rs['adjective'],
        Rs['adjective'],
        R('adjective', defaults=[('QUIET', 0.4)], postproc=skipart)),
    Meme('SO LONG AND THANKS FOR ALL THE {}',
        Rs['noun plural mass property']),
    Meme('SOMETIMES YOU {0} THE {1}, SOMETIMES THE {1} {2} YOU',
        Rs['verb tr inf'],
        Rs['noun singular'],
        IndexMap(0, lambda x: x.wordforms[v_present])),
    Meme('SPEAK {} AND CARRY {} {}',
        R('adverb', defaults=[('SOFTLY', 0.75)]),
        R('adjective', defaults=[('A SEAISH', 0.33)]),
        Rs['noun singular']),
    Meme('SUDDENLY, {}. THOUSANDS OF THEM!',
        Rs['noun plural']),
    Meme('THAT IS NOT {} WHICH CAN {} LIE / AND WITH {} {} EVEN {} MAY DIE',
        Rs['adjective'],
        R('adjective', defaults=[('ETERNAL', 0.5)], postproc=skipart),
        R('adjective', defaults=[('STRANGE', 0.5), ('EPSILONISH', 0.2)],
            postproc=skipart),
        R(('noun-interval', n_plural), defaults=[('AEONS', 0.67)]),
        IndexMap(0, lambda x: skipword(x.wordforms[a_property]))),
    Meme('THAT\'S ONE RIVERISH {0} FOR {1}, ONE SEAISH {0} FOR {2}KIND',
        Rs['noun singular'],
        Rs['noun singular article'],
        IndexMap(1, skipword)),
    Meme('THE FIRST RULE OF {0} IS, YOU DON\'T TALK ABOUT {0}',
        R('noun', 'noun-unique', 'noun-mass', ('adjective', a_property),
            postproc=lambda x: random.choice(('THE {}', '{} CLUB')).format( \
                skipword(x)) if x.originclass == 'noun-unique' \
                else ((skipword(x) + random.choice(('', ' CLUB'))) \
                    if x.originclass == 'noun-mass' \
                    else (skipart(x) + ' CLUB')))), # Credit: mrob27
    Meme('THERE IS NO {}{} FOR THE {}',
        Rs['noun singular mass property'],
        C((', SAID THE GLR,', 0.25), (', SAID RANDALL,', 0.33), ('', 1.0)),
        Rs['adjective']), # Credit: mrob27
    Meme('THERE\'S NO {} {}',
        C(('E**ISHNESS', 0.33), ('SADNESS', 0.5), ('CRYING', 1.0)),
        Rs['place prep']), # Credit: mrob27
    Meme('THESE AREN\'T THE {} YOU\'RE {} FOR',
        Rs['noun plural'],
        Rs['verb cont']), # Credit: mrob27
    Meme('THEY\'RE TAKING THE {} TO {}!',
        R(('noun', n_plural), defaults=[('MOLPIES', 0.1)]),
        R('place', defaults=[('ISENGARD', 0.33), ('ᘝᓄᘈᖉᐣ', 0.33)])),
            # Credit: mrob27
    Meme('THE {} IS TOO CH*RPING {}',
        Rs['noun singular'],
        R('adjective', defaults=[('HIGH', 0.3)], postproc=skipart)),
            # Credit: mrob27
    Meme('THE {} PART OF {} IS {} IN YOUR {}',
        R(('adjective', a_super), defaults=[('MOLPIEST', 0.25)]),
        R(('verb-intr', v_cont), ('verb-tr', v_cont),
            defaults=[('MOLPYING UP', 0.2)]),
        Rs['noun plural mass property'],
        Rs['noun singular mass']), # Credit: mrob27
    Meme('THE {} {} HERE',
        Rs['noun singular'],
        R(('verb-intr', v_present), defaults=[('STOPS', 0.70),
            ('**DS', 0.33)])), # Credit: balthememe and BlitzGirl
    Meme('TO {0} OR NOT TO {0}, THAT IS THE QUESTION',
            Rs['verb inf']), # Credit: mrob27
    Meme('WE\'RE GONNA NEED A MORE {} {}',
        R('adjective', defaults=[('SEAISH', 0.5)], postproc=skipart),
        Rs['noun singular']), # Credit: mrob27
    Meme('WHAT HAPPENS {0}, STAYS {0}',
        Rs['place prep']),
    Meme('WHAT! NO, NOT THE {} _{}_!',
        R('noun', 'noun-mass', defaults=[('MIND', 0.5)], postproc=skipart),
        IndexMap(0, lambda x: Rs['noun singular mass'].pick() \
            if x.stringform == 'MIND' else 'PROBE')),
    Meme('WITH {} LIKE THESE, WHO NEEDS {}?',
        Rs['noun plural'],
        Rs['noun plural mass property']),
    Meme('WOW - SO {} - MANY {} - SUCH {} - VERY {} - MUCH {} - {}',
        Rs['noun singular'],
        Rs['adjective'],
        Rs['verb cont'],
        Rs['noun singular'],
        Rs['adjective'],
        C(('SEA IS BIG', 0.33), ('RIVER IS SMALL', 0.5),
            ('HAIL RANDALL', 1.0))), # Credit: mrob27
    Meme('...XE\'S {}A FEW {} {} OF {}',
        C(('JUST ', 0.5), ('', 1.0)),
        Rs['noun plural'],
        C(('SHY', 0.5), ('SHORT', 1.0)),
        Rs['noun singular article']), # Credit: mrob27
    Meme('...XE WASN\'T THE {} {} {}',
        Rs['adjective superlative'],
        Rs['noun commodity'],
        Rs['place prep']), # Credit: mrob27
    Meme('Y\'ALL GOT ANY MORE OF THEM {}?',
        Rs['noun plural']),
    Meme('YES {}, THERE _IS_ {}.',
        R('character', defaults=[('VIRGINIA', 0.2)], postproc=skipthe),
        R('noun-unique', 'noun-unique', 'noun')), # Credit: mrob27
    Meme('YOU ARE {} {}, ALL {}',
        cavememe,
        Rs['noun plural mass'],
        Rs['adjective']), # Credit: mrob27
    Meme('YOU CAN TAKE {0} OUT OF {1}, BUT YOU CAN\'T TAKE {1} OUT OF {0}',
        R('noun', 'noun-unique', 'character', postproc=lambda x: \
            ('THE ' + skipart(x)) if x.originclass.startswith('noun') \
            else x.stringform),
        R('place', selector=lambda x: x.wordforms[1] != 'ON')),
    Meme('YOU CAN\'T {} YOUR {} AND {} IT TOO',
        Rs['verb tr inf'],
        Rs['noun singular mass'],
        Rs['verb tr inf']), # Credit: mrob27
    Meme('YOU GET {0}! AND YOU GET {0}! AND YOU GET {0}! '
            '_EVERYMOLPY GETS {0}!_',
        Rs['noun singular article mass']),
    Meme('YOUR {0}\'S {1} AND YOU SHOULD FEEL {1}',
        Rs['noun singular'],
        Rs['adjective negative'] << C(('BAD', 0.33))),
    Meme('YOU {0} BY THE {1}, YOU {2} BY THE {1}',
        R('verb-intr', 'verb-tr', defaults=[('LIVE', 0.3)]),
        R('noun', defaults=[('PUNSAW', 0.25), ('THWAPSTICK', 0.3)],
            postproc=skipart),
        R('verb-intr', 'verb-tr', defaults=[('DIE', 0.3)])), # Credit: mrob27
    Meme('{}. *NOT TO BE CONFUSED WITH _{}_*',
        R('noun', 'noun-unique', 'noun-mass', ('adjective', a_property),
            ('verb-intr', v_cont), ('verb-tr', v_cont), postproc=skipart),
        IndexMap(0, lambda x: Rs['verb cont'].pick() \
            if x.originclass.startswith('verb') \
            else Rs['noun singular mass property'].pick())),
            # Credit: azule via mrob27
    Meme('_{} 2: ELECTRIC BAGALOO_',
        R(('noun', n_singular, n_plural), ('verb-tr', v_cont), 'noun-mass',
            ('adjective', a_property), postproc=skipart)), # Credit: mrob27
    Meme('{0} {1} LET {0} {2} {3}',
        R(('noun', n_plural), defaults=[('FRIENDS', 0.85)]),
        C(('DON\'T', 0.9), ('_DO_', 1.0)),
        Rs['verb tr inf'],
        Rs['noun plural mass property']),
    Meme('{}HAST {} THE {} {}?',
        C(('RAFTCASTLE AHOY! ', 0.2), ('', 1.0)),
        R(('verb-tr', v_perfect), ('verb-tr-only', v_perfect),
            defaults=[('SEEN', 0.67)]),
        Rs['adjective'],
        R('noun', defaults=[('WHALPY', 0.33)], postproc=skipart)),
    Meme('_{0} HARD 2: {0} HARDER_',
        Rs['verb inf 1word']),
    Meme('{} AND {} AND {}, OH MY!',
        Rs['noun plural'],
        Rs['noun plural'],
        Rs['noun plural mass property']), # Credit: mrob27
    Meme('{} AND {}, SITTING IN A {}TREE, {}',
        Rs['character'],
        Rs['character'],
        C(('NEAT ', 0.33), ('WOW', 0.5), ('', 1.0)),
        R(('verb-intr', v_cont), ('verb-tr', v_cont),
            selector=lambda x: x.stringform.endswith('ING'),
            postproc=lambda x: '-'.join(x.stringform))),
    Meme('{0} DON\'T CARE. {0} DON\'T GIVE A CH*RP.',
        Rs['noun singular mass character skipthe']),
    Meme('{} FOR THE {} GOD! {} FOR THE {} THRONE!',
        Rs['noun plural mass property'],
        IndexMap(0, lambda x: skipword(x.wordforms[n_singular]) \
                if x.originclass == 'noun' else x.stringform),
        Rs['noun plural mass'],
        IndexMap(2, lambda x: skipword(x.wordforms[n_singular]) \
                if x.originclass == 'noun' else x.stringform)),
    Meme('{} GOES WHERE‽',
        Rs['noun singular']),
    Meme('{} GRAB MY {}',
        Rs['character skipthe'],
        Rs['noun singular plural']),
    Meme('{}{} HAPPENS.',
        R('adjective', defaults=[('', 0.5)],
            postproc=lambda x: skipart(x) + ' '),
        Rs['noun singular mass property']), # Credit: mrob27
    Meme('{} HAVE {} WORDS FOR {}, BUT NONE FOR {}',
        Rs['noun plural'],
        lambda: random.choice(['OVER 27', 'ABOUT A HUNDRED',
            str(random.randint(4, 64))]),
        R(('noun', n_plural), ('verb-intr', v_cont), ('verb-tr', v_cont),
            'noun-mass', postproc=skipart),
        R(('noun', n_plural), ('verb-intr', v_cont), ('verb-tr', v_cont),
            'noun-mass', postproc=skipart)), # Credit: mrob27
    Meme('{}... IN SPACE!',
        Rs['noun plural mass character']),
    Meme('{} IS A CUTE {}',
        Rs['noun singular character'],
        IndexMap(0, skipthe)), # Credit: mrob27
    Meme('{} IS A {}',
        R(('verb-intr', v_cont), ('verb-tr', v_cont),
            selector=lambda x: x.stringform.endswith('ING')),
        R('noun', defaults=[('ART', 0.85)], selector=lambda x: \
            x.stringform.startswith('AN '), postproc=skipart)),
    Meme('{} IS A TIME LORD AND {} IS XIS TARDIS',
        Rs['character'],
        R('noun', 'noun-unique', 'place', postproc=lambda x: \
            ('THE ' + skipart(x)) if x.originclass.startswith('noun') \
            else x.stringform)),
    Meme('{} IS HARD, LET\'S GO {}.',
        Rs['verb cont'],
        Rs['verb cont']), # Credit: mrob27
    Meme('{} IS THE {} {} DESERVE{}, BUT NOT THE ONE {} RIGHT NOW.',
        Rs['character'],
        R('noun', 'noun-unique', defaults=[('HERO', 0.3)], postproc=skipart),
        Rs['place'],
        IndexMap(2, lambda x: 'S' if x.wordforms[2] != 'SOME' else ''),
        IndexMap(2, lambda x: 'THEY NEED' if x.wordforms[2] == 'SOME' \
            else 'IT NEEDS')),
    Meme('{} IS {}SO _LAST {}_',
        Rs['verb cont'],
        C(('JUST ', 0.5), ('', 1.0)),
        C(('NEWPIX', 0.2), ('DIP', 0.25), ('XIP', 0.33),
            ('MIP', 0.5), ('YIP', 1.0))), # Credit: mrob27
    Meme('{} IS {}. {}? NOT SO MUCH.',
        Rs['noun singular article mass character property'],
        Rs['adjective'],
        Rs['noun singular article mass character property']),
    Meme('{} IS "{} {}"... IF YOU KNOW WHAT I MEAN',
        Rs['character'],
        Rs['verb tr cont'],
        R('noun', 'noun-unique', 'place', 'character',
            postproc=lambda x: ('THE ' + skipart(x)) \
                if x.originclass == 'noun-unique' \
                else (('THE ' + skipart(x)) \
                    if x.originclass == 'noun' \
                    and random.choice((True, False)) else x.stringform))),
            # Credit: mrob27 and azule
    Meme('{} IT FROM {}. IT\'S THE ONLY WAY TO BE {}.',
        Rs['verb tr inf'],
        R('place', defaults=[('ORBIT', 0.85)]),
        R('adjective', defaults=[('SURE', 0.85)], postproc=skipart)),
    Meme('{}, I\'VE GOT A FEELING WE\'RE NOT {} ANYMORE.',
        Rs['character skipthe'],
        Rs['place prep']),
    Meme('{} LIKE {}, {} LIKE {}',
        Rs['verb inf 1word'],
        R('noun', defaults=[('A FLUTTERBEE', 0.9)]),
        Rs['verb inf 1word'],
        R('noun', defaults=[('A BEESNAKE', 0.9)])),
    Meme('{}! LOOK AT THE SIZE OF {} {}!',
        C(('CRIKEY', 0.45), ('CROIKEY', 0.82), ('BAOBABS', 1.0)),
        IndexMap(2, lambda x: 'THOSE' if x.wordform == n_plural else 'THAT'),
        R(('noun', n_singular, n_plural), 'noun-unique', postproc=skipart)),
    Meme('{0}MAN, {0}MAN, DOES WHATEVER {1} CAN',
        IndexMap(1, skipword),
        Rs['noun singular article']),
    Meme('{} ME YOUR {}, YOUR {}, YOUR {} {} YEARNING TO {} {}',
        C(('BRING', 0.5), ('GIVE', 1.0)),
        Rs['adjective'],
        Rs['adjective'],
        Rs['adjective'],
        Rs['noun plural'],
        Rs['verb inf'],
        IndexMap(5, lambda x: Rs['noun plural mass'].pick() \
            if x.originclass == 'verb-tr' \
            else random.choice(('FREE', 'MOLPISHLY')))), # Credit: mrob27
    Meme('{} POP LIKE {} {}. SPLASHING {} IN {}. I AM {}.',
        Rs['noun plural'],
        R('adjective', defaults=[('VIBRANT', 0.15)], postproc=skipart),
        R(('noun', n_plural), defaults=[('SEEDS', 0.25)]),
        Rs['noun plural'],
        R('place', defaults=[('A VOID', 0.75)], selector=lambda x:
            x.wordforms[1] == 'IN'),
        R('adjective', defaults=[('STICKY', 0.25)], postproc=skipart)),
    Meme('{}, RELEASE THE {}!',
        R('character', defaults=[('SMITHERS', 0.5)], postproc=skipthe),
        Rs['noun plural']),
    Meme('{} RUN WHEN {} {} GOES TO {}',
        Rs['noun plural'],
        R('adjective', defaults=[('A GOOD', 0.25), ('A MOLPISH', 0.33)]),
        Rs['noun commodity'],
        R('place', defaults=[('WAR', 0.33)])),
    Meme('{}: THE FINAL FRONTIER. THESE ARE THE VOYAGES OF THE STARSHIPCASTLE'
            ' _{}_. ITS CONTINUING MISSION: TO EXPLORE {} NEW WORLDS, TO SEEK'
            ' OUT NEW {} AND NEW {}, TO {} {} WHERE NO {} HAS {} BEFORE.',
        Rs['place'],
        Rs['noun singular mass property'],
        Rs['adjective'],
        Rs['noun mass property'],
        Rs['noun plural'],
        Rs['adverb'],
        Rs['verb inf'],
        R('noun', defaults=[('MOLPY', 0.5)], postproc=skipart),
        IndexMap(6, lambda x: x.wordforms[v_perfect])),
    Meme('{0} THE {1}, {0} THE WORLD',
        R('verb-tr', 'verb-tr-only', defaults=[('SAVE', 0.67)]),
        Rs['noun singular plural mass']),
    Meme('{} THE {} {}! {} THE {}! UNIMAGINABLE {}! UNLIMITED {}! '
        'ET CETERA, ET CETERA!',
        Rs['verb tr inf'],
        R(('adjective', a_comp), defaults=[('LESSER', 0.67)],
            postproc=skipart),
        Rs['noun plural'],
        Rs['verb tr inf'],
        Rs['noun singular mass'],
        Rs['noun mass property'],
        R(('noun', n_plural), 'noun-mass', defaults=[('RICE PUDDING', 0.9)],
            postproc=skipart)),
    Meme('{} USED {}! {}',
        Rs['character'],
        Rs['verb inf'],
        C(('IT\'S SUPER EFFECTIVE!', 0.5), ('IT\'S NOT VERY EFFECTIVE...',
            0.7), ('...BUT IT FAILED!', 1.0))),
    Meme('{} WALKS DOWN {} IN THAT {}, '
            'PEOPLE KNOW {}\'S NOT AFRAID OF ANYTHING',
        C(('MAN', 0.25), ('WOMAN', 0.33), ('MOLPY', 0.5), ('OTTER', 1.0)),
        Rs['place'],
        Rs['noun commodity'],
        IndexMap(0, lambda x: 'HE' if x.stringform == 'MAN' else ( \
            'SHE' if x.stringform == 'WOMAN' else 'XE'))),
    Meme('{0}? WE DON\'T NEED NO {1} {0}.',
        Rs['noun plural mass property'],
        R(('verb-intr', v_cont), ('verb-tr', v_cont),
            defaults=[('CH*RPIN\'', 0.33), ('STINKIN\'', 0.5)],
            selector=lambda x: x.stringform.endswith('ING') \
                and len(x.stringform.split()) == 1,
            postproc=lambda x: x.stringform[:-1] + '\'')), # Credit: mrob27
    Meme('{}, WE HAVE {}SIGN THE LIKES OF WHICH EVEN {} HAS NEVER SEEN!',
        Rs['character'],
        R('noun', 'noun-unique', 'noun-mass', ('adjective', a_property),
            selector=lambda x: len(x.stringform.split()) == \
                    (2 if x.originclass == 'noun' else 1), postproc=skipart),
        C(('RANDALL', 0.33), ('THE GLR', 0.5), ('THE OTA', 1.0))),
    Meme('{0}, {0} AGAINST THE {1} OF THE {2}',
        R('verb-tr', defaults=[('RAGE', 0.75)]),
        Rs['verb cont'],
        Rs['noun singular mass property']),
    Meme('♫ {} {} ALIVE WITH THE SOUND OF {} ♫',
        Rs['place'],
        IndexMap(0, lambda x: 'ARE' if x.wordforms[2] == 'SOME' \
            else 'IS'),
        R(('noun', n_plural), 'noun-mass', ('adjective', a_property),
            ('verb-intr', v_cont), ('verb-tr', v_cont), postproc=skipart)),
    Meme('{}, {} CAN {} YOU {}',
        Rs['place prep'],
        C(('NO-ONE', 0.5), ('NOMOLPY', 1.0)),
        C(('SEE', 0.5), ('HEAR', 1.0)),
        Rs['verb inf']), # Credit: mrob27
    Meme('{} {} GONNA {}',
        R(('verb-intr', v_cont), ('verb-tr', v_cont),
            selector=lambda x: x.stringform.endswith('ING')),
        IndexMap(0, lambda x: erify(x.stringform) + 'S'),
        IndexMap(0, lambda x: x.wordforms[v_inf])), # Credit: mrob27
    Meme('{} {} GROW ON {}.',
        Rs['noun plural mass'],
        IndexMap(0, lambda x: 'DOESN\'T' if x.originclass == 'noun-mass' \
            else 'DON\'T'),
        R(('noun', n_plural), defaults=[('TREES', 0.25), ('BAOBABS', 0.33)])),
            # Credit: azule via mrob27
    Meme('{0}, {0}, {4}-{5}! THESE ARE THE {1} WORDS THAT {2} '
            'CREATED XIMSELF. DON\'T {3} THEM!',
        IndexMap(2, skipthe),
        R('adjective', defaults=[('MAGIC', 0.75)], postproc=skipart),
        R('character', selector=lambda x: len(x.stringform.split()) == 1 \
            or (x.stringform.startswith('THE ') \
                and len(x.stringform.split()) == 2)),
        R('verb-tr', defaults=[('STEAL', 0.25)]),
        edname(2, ('KOOLOO', 0.2)),
        edname(2, ('LIMPAH', 0.2))),
    Meme('{0}, {0}, MY KINGDOM FOR {0}',
            Rs['noun singular article']), # Credit: mrob27
    Meme('{} {} THE NEW {}',
        Rs['noun singular plural mass property'],
        IndexMap(0, lambda x: 'ARE' if x.wordform == n_plural else 'IS'),
        IndexMap(0, lambda x: Rs['noun plural'].pick().stringform \
            if x.wordform == n_plural \
            else Rs['noun singular mass property'].pick().stringform)),
            # Credit: mrob27
    Meme('{0}{1}THE {2} {3} WERE NEITHER {2} NOR {3}. DISCUSS.',
        C(('TAWK AMONGST YOA-XELVES. ', 0.5), ('', 1.0)),
        C(('I\'LL GIVE YOO A TOAPIC... ', 0.5), ('', 0.5)),
        Rs['adjective'],
        Rs['noun plural']), # Credit: mrob27
    Meme('{0}{1}THE {2} {3} {4} WAS NEITHER {2} NOR {3} NOR {5}. DISCUSS.',
        C(('TAWK AMONGST YOA-XELVES. ', 0.5), ('', 1.0)),
        C(('I\'LL GIVE YOO A TOAPIC... ', 0.5), ('', 0.5)),
        Rs['adjective'],
        Rs['adjective'],
        Rs['noun singular'],
        IndexMap(4, lambda x: x.wordforms[n_singular])), # Credit: mrob27
    Meme('{} - {} {}',
        C(('404', 0.33), ('408', 0.5), ('503', 1.0)),
        Rs['noun singular'],
        IndexMap(0, lambda x: {'404' : 'NOT FOUND', '408': 'REQUEST TIMEOUT',
            '503' : 'UNAVAILABLE'}[x.stringform])),
    Meme('{0} {1}: _{1}!_ AND THE {2} IS GONE!',
        R('noun', 'noun-mass', 'noun-unique', 'character', 'place',
            defaults=[('MOLPIT', 0.4), ('MOLPY', 0.17)],
            selector=lambda x: (len(x.stringform.split()) == 2) \
                if x.originclass.startswith('noun') \
                or x.stringform.startswith('THE ') \
                else (len(x.stringform.split()) == 1),
            postproc=curry(skipthe, skipart)),
        C(('BANG', 0.75), ('THWAP', 0.5), ('BONG', 0.5), ('ONG', 1.0)),
        R('noun-mass', ('adjective-negative', a_property), postproc=skipart))
]

memes_meta_molpish = [
    Meme('DID ANYONE NOTICE THE {}?',
        Rs['noun singular plural mass']), # Credit: azule
    Meme('IF WE HAVE {0} YOU FOR SOME {1}, PLEASE {2} THAT WE GOT ALL {1} WE'
            ' {3}, OR {4} TO US THAT YOU WON\'T {5} OUR {6} WE {3} THE {1}'
            ' FOR.',
        Rs['verb tr past'],
        R(('noun', n_plural), 'noun-mass', defaults=[('DATA', 0.2)],
            postproc=skipart),
        R('verb-intr', defaults=[('CONFIRM', 0.5)],
            postproc=lambda x: ('RE' + x.stringform) \
                if x.stringform[0] not in string.punctuation \
                else (x.stringform[0] + 'RE' + x.stringform[1:])),
        R(('verb-tr', v_inf, v_perfect), defaults=[('NEED', 0.3)]),
        R('verb-intr', 'verb-tr', defaults=[('CONFIRM', 0.2)]),
        R('verb-tr', defaults=[('USE', 0.2)]),
        R(('noun', n_singular, n_plural), 'noun-mass',
            defaults=[('SERVICES', 0.2)], postproc=skipart)), # Credit: mrob27
    Meme('IF {} IS THE {} REPRESENTATION OF {}, WHO IS THE {} '
            'EQUIVALENT OF {}?',
        Rs['character'],
        R('place', 'noun-mass', defaults=[('BFTF', 0.2)], postproc=lambda x: \
                skipthe(x) if x.originclass == 'place' else (skipword(x) \
                if x.originclass != 'x-defaults' else x)),
        Rs['character'],
        R('place', 'noun-mass', defaults=[('OTT', 0.2)], postproc=lambda x: \
                skipthe(x) if x.originclass == 'place' else (skipword(x) \
                if x.originclass != 'x-defaults' else x)),
        Rs['character']), # Credit: ZoomanSP via mrob27
    Meme('I {} HALF {} FOR ONE {} {}',
        R(('verb-tr', v_past), defaults=[('WAITED', 0.6)]),
        C(('AN HOUR', 0.1), ('A DIP', 0.25), ('A NOPIX', 0.33),
            ('A MIP', 0.5), ('A YIP', 1.0)),
        R(('verb-intr', v_cont), ('verb-tr', v_cont),
            defaults=[('DARKENING', 0.7)]),
        R('noun', defaults=[('PIXEL', 0.5)], postproc=skipart)),
        # Credit: azule
    Meme('MAYBE {} {}! MAYBE {} IS A METAPHOR FOR {}!',
        Rs['character'],
        C(('IS PREGNANT', 0.2), ('HAS CANCER', 0.25), ('HAD AN ABORTION',
            0.33), ('GOT MARRIED', 0.5), ('GOT DIVORCED', 1.0)),
        R('noun', 'noun-unique', 'noun-mass', defaults=[('THE OTC', 0.2)],
            postproc=lambda x: ('THE ' + skipart(x)) \
                if x.originclass != 'noun-mass' else skipart_mup(x)),
        R(('noun', 1), 'noun-mass', defaults=[('THIS', 0.1), ('THAT', 0.1)],
            postproc=skipart)),
    Meme('NI NI {} {} {}',
        C(('NI', 0.5), ('NI NI', 0.5), ('NI NI NI', 1.0)),
        R('noun', 'noun-unique', 'noun-mass', ('adjective', a_property),
            defaults=[('CHUPACABRA', 0.1)], postproc=skipart),
        R('noun', 'noun-unique', 'noun-mass', ('adjective', a_property),
            defaults=[('PING PONG BALL', 0.1)], postproc=skipart)),
    Meme('OT{}: ONE TRUE {}',
        IndexMap(1, lambda x: ''.join([y[0] for y in x.stringform.split()])),
        Rs['noun singular']), # Credit: azule
    Meme('REDUNDANT {} IS REDUNDANT',
        Rs['noun singular mass property']), # Credit: azule via mrob27
    Meme('¡THIS {} IS {} ME!',
        Rs['noun singular mass property'],
        Rs['verb tr cont']),
    Meme('WELL, ANYTHING _COULD_ BE {}, INCLUDING THIS {} I JUST {}.',
        R('noun', defaults=[('A MEME', 0.2)]),
        R('noun', 'noun-mass', defaults=[('SENTENCE', 0.2)],
            postproc=skipart),
        Rs['verb tr past']), # Credit: azule and mrob27
    Meme('{} ARE {} WHO {} {} INLAND.',
        Rs['noun plural'] << C(('BANANAS', 0.2), ('BANANANAS', 0.2)),
        Rs['noun plural'],
        Rs['verb tr inf'],
        Rs['noun plural mass']),
    Meme('"{} IT IN YOUR {}" SOUNDS LIKE SLANG FOR TELLING SOMEONE TO {}.',
        Rs['verb tr inf'],
        Rs['noun singular mass'],
        Rs['verb inf'] << C(('GTCO', 0.2), ('CH*RP OFF', 0.25))),
        # Credit: lmjb1964 via mrob27
    Meme('"{} THE {}" SOUNDS LIKE {} {}.',
        Rs['verb tr inf'],
        Rs['noun singular plural mass'],
        R('adjective', defaults=[('A FUN', 0.3), ('A DUNEJUMPISH', 0.43),
            ('A MOLPISH', 0.4)]),
        R('noun', defaults=[('HOTDOG', 0.4), ('GAME', 0.6)],
            postproc=skipart)), # Credit: HES and mrob27
    Meme('{0} USED EMERGENCY {1}! IT\'S A SUPER {1}!',
        R('character', defaults=[('LINK', 0.2)]),
        R(('noun', n_singular), defaults=[('KITTEN', 0.2), ('LINK', 0.25)],
            postproc=skipart)), # Credit: mrob27
    Meme('{} {} IS {} THE {} {} ALBUM',
        Rs['adjective'],
        R(('noun', n_singular, n_plural), 'noun-mass', ('verb-intr', v_cont),
            ('verb-tr', v_cont), ('adjective', a_property), postproc=skipart),
        C(('A HIDDEN TRACK ON', 0.25), ('THE NAME OF', 0.33),
            ('THE LATEST SINGLE OFF OF', 1.0)),
        C(('LATEST', 0.3), ('NEWEST', 1.0)),
        C(('VHF', 0.5), ('VITAL HOTDOG FUNCTION', 1.0))),
        # Credit: azule and mrob27
    Meme('{} {} {}',
        R(('adjective', a_property), ('noun', n_plural), 'noun-mass',
            defaults=[('REDUNDANCY', 0.2), ('REDUNDAKITTY', 0.1)],
            postproc=skipart),
        IndexMap(0, lambda x: 'ARE' if x.originclass == 'noun' else 'IS'),
        R('adjective-positive', defaults=[('MOLPISH', 0.5),
            ('MOLPFISH', 0.1)], postproc=skipart)), # Credit: azule via mrob27
    Meme('{}. {}. {}. {}. {}.',
        R('noun', defaults=[('GAZEBO', 0.33)], postproc=skipart),
        R('noun-mass', ('adjective', a_property),
            defaults=[('OINTMENT', 0.33)], postproc=skipart),
        R('noun', defaults=[('HARPSICHORD', 0.33)], postproc=skipart),
        R('noun', defaults=[('CREDENZA', 0.33)], postproc=skipart),
        R('noun', defaults=[('BUNGALOW', 0.33)], postproc=skipart)),
    Meme('{0} {0} {0} {0} {0} {0} {0} {0} {0} {0} {0} {0} {1} {1}',
        R('noun', 'noun-unique', 'noun-mass', ('adjective', a_property),
            defaults=[('MOLPY', 0.1)], selector=lambda x: \
                len(x.stringform.split()) == (2 if x.originclass == 'noun' \
                else 1), postproc=skipart),
        R('noun', 'noun-unique', 'noun-mass', ('adjective', a_property),
            defaults=[('GRAPEVINE', 0.1)], selector=lambda x: \
                len(x.stringform.split()) == (2 if x.originclass == 'noun' \
                else 1), postproc=skipart)) # Credit: mrob27
]


memes_ottomeme = [ # Memes from OTToMeme by azule
                   # http://xkcd.modularthought.com/ottomeme/
                   # Full credit to azule for all memes in this category
                   # All memes here were taken from mrob27's list
    Meme('THE {} {} HAPPENED BEFORE I WAS BORN, YET I\'M OLD ENOUGH TO HAVE '
            'THIS CONVERSATION WITH YOU.',
        R('adjective-negative', defaults=[('ZANCLEAN', 0.5)],
            postproc=skipart),
        IndexMap(0, lambda x: Rs['noun singular mass'].pick() \
                if x.originclass == 'x-defaults' else 'FLOOD')),
    Meme('{} {} CREATES AN UNRESOLVED TENSION THAT WILL STAY WITH YOU ALL {}',
        Rs['adjective article'],
        Rs['noun singular'],
        Rs['noun interval']),
    Meme('CAN YOU TRY TO LOOK... {}?',
        R(('adjective', a_comp), postproc=skipart)),
    Meme('[{} {}]',
        R('noun', 'noun-mass', defaults=[('CITATION', 0.5)],
            postproc=skipart),
        IndexMap(0, lambda x: Rs['verb tr past'].pick() \
                if x.originclass == 'x-defaults' \
                    or random.random() > 0.7 else 'NEEDED')),
    Meme('{}! I JUST GAVE {} CANCER.',
        R('adjective-positive', defaults=[('TREEISH', 0.25),
            ('MOLPISH', 0.25), ('NEAT', 0.3)], postproc=skipart),
        R('noun', 'noun-mass', 'noun-unique', 'place', postproc=skipart_mup)),
    Meme('EASY-TO-REMEMBER PASSWORD: {} {} {} {}',
        R('adjective', defaults=[('CORRECT', 0.2)], postproc=skipart),
        Rs['character skipthe'],
        Rs['noun singular mass property'],
        Rs['noun singular mass property']),
    Meme('{}, STAND BACK. I KNOW {}{}.',
        C(('EVERYBODY', 0.5), ('EVERYMOLPY', 1.0)),
        IndexMap(2, lambda x: (str(Rs['adjective'].pick()) + ' ') \
                if x.originclass == 'noun' else ''),
        R(('noun', n_plural), ('verb-tr', v_cont), ('verb-intr', v_cont),
            postproc=skipart)),
    Meme('I DON\'T THINK {} {} COUNTS AS {}.',
        Rs['verb tr cont'],
        Rs['noun singular article'],
        R('adjective-positive', defaults=[('SEXY', 0.3)], postproc=skipart)),
    Meme('I DON\'T KNOW WHAT YOU JUST SAID BECAUSE I WAS THINKING ABOUT {}',
        R(('verb-intr', v_cont), ('noun', n_plural), 'noun-mass',
            'noun-unique', 'place', postproc=lambda x:
                ('THE ' if x.originclass == 'noun-unique' else '') + \
                skipart(x))),
    Meme('I FEEL PARALYZED BY {}. ...AND YET FOR SOME REASON '
        'I\'M _REALLY_ EXCITED ABOUT {}?',
        C(('OVERWHELMING EXISTENTIAL DREAD', 0.33), ('THE SADNESS', 0.5),
            ('T** **D', 1.0)),
        R(('verb-intr', v_cont), ('verb-tr', v_cont), ('noun', n_plural),
            'noun-mass', postproc=skipart)),
    Meme('IF WE MAKE IT BACK ALIVE, YOU\'RE NEVER {} ANYTHING AGAIN.',
        Rs['verb tr cont']),
    Meme('IF YOU {0} {1}, YOU {0} IN _REAL LIFE_!',
        R('verb-intr', defaults=[('DIE', 0.33)]),
        Rs['place prep']),
    Meme('I KNOW IT\'S NATURAL AND ALL, '
            'BUT I REALLY WISH {} {} WOULDN\'T TRY TO {} ME.',
        Rs['noun plural'],
        Rs['place prep'],
        Rs['verb tr inf']),
    Meme('I _LIKE_ MY {}',
        R('noun', 'noun-mass', 'noun-unique', defaults=[('HAT', 0.15)],
            postproc=skipart)),
    Meme('I\'M {}.  I... I DIDN\'T KNOW.',
        Rs['noun singular article']),
    Meme('I\'M DEVELOPING A SYSTEM TO {} ARBITRARY {}.',
        R('verb-tr', defaults=[('PASS YOU', 0.3)]),
        R(('noun', n_plural), defaults=[('CONDIMENTS', 0.2)])),
    Meme('I\'M NOT REALLY INTO {}.',
        Rs['noun plural mass']),
    Meme('IN MY {}, PEOPLE _{}_.',
        R('noun', 'noun-unique', 'noun-mass', defaults=[('PORN', 0.3),
            ('SLASH-FICTION', 0.2)], postproc=skipart),
        Rs['verb inf']),
    Meme('{}{} CONTAINED {}. WOULD {}{} AGAIN.',
        IndexMap(3, lambda x:('INSTEAD OF ' + \
                str(Rs['noun singular mass'].pick()) + ', ') \
            if x.stringform == 'NOT ' \
            else ''),
        R('noun', defaults=[('PACKAGE', 0.5)], postproc=skipart),
        Rs['noun singular mass'],
        C(('NOT ', 0.5), ('', 0.5)),
        R('verb-tr', defaults=[('BUY', 0.5)])),
    Meme('IT CAN DO WHATEVER IT WANTS. IT\'S {}.',
        R('noun', 'noun-mass', 'noun-unique', defaults=[('THE SEA', 0.2)],
            postproc=lambda x: ('THE ' if x.originclass != 'noun-mass' \
                else '') + skipart(x))),
    Meme('I WONDER WHERE I\'LL {} NEXT?',
        Rs['verb inf']),
    Meme('_LISTEN UP!_  THE FIRST RULE OF {0} CLUB '
            'IS THE FIRST RULE OF {0} CLUB.',
        R('noun', 'noun-mass', 'noun-unique', ('adjective', a_property),
            defaults=[('REDUNDANCY', 0.2)], postproc=skipart)),
    Meme('NONE OF THE PLACES I {} HAD {}.',
        R(('verb-tr', v_past), defaults=[('FLOATED', 0.5)]),
        Rs['noun plural']),
    Meme('...ooOOoooOOoo... {} IS {}!',
            R('noun', 'place', postproc=curry(lambda x: x.replace('þ', 'THE'),
                curry(aaaagile, curry(lambda x: x.replace('THE ', 'þ '),
                skipart)))),
        R(('verb-intr', v_cont), ('verb-tr', v_cont), postproc=aggiiiile)),
    Meme('OUR COPY OF THE {} HAS A COUPLE MISTRANSLATIONS. '
            'WHICH WE REFUSE TO FIX.',
        Rs['noun singular']),
    Meme('{}: THEY ARE ABOUT THIS {}. NOW CAN WE _PLEASE_, AS {}, MOVE ON?',
        Rs['noun plural'],
        R('adjective', defaults=[('BIG', 0.25), ('SEAISH', 0.33)],
            postproc=skipart),
        C(('A CULTURE', 0.25), ('THE OTT', 0.33), ('A THREAD', 0.5),
            ('A NEEDLE-PULLED THING', 1.0))),
    Meme('{} {} {} ...DOT CHIRPING MUSTARD DOT COM.',
        Rs['noun singular'],
        Rs['verb tr cont'],
        R('noun-unique', defaults=[('DUNGEON', 0.3)],
            postproc=lambda x: 'THE ' + skipart(x))),
    Meme('{}_REAL_ {} USE {}.{}',
        C(('EXCUSE ME, BUT ', 0.5), ('', 1.0)),
        Rs['noun plural'],
        Rs['noun singular article plural mass'],
        IndexMap((0, 2), lambda x, y: (' OH YEAH! GOOD OL\' C-X M-C M-' + \
            (skipword(y) if y.originclass == 'noun' \
            and y.wordform == n_singular else y.stringform) + '...') \
            if len(x.stringform) > 0 else '')),
    Meme('{}. IT WORKS, CH*RPIES.',
        R('noun', 'noun-mass', ('verb-intr', v_cont), ('verb-tr', v_cont),
            postproc=lambda x: ('THE ' if x.originclass == 'noun' else '') + \
                skipart(x))),
    Meme('{} AND {} CREATE {} IN LAB: "THE TRICK WAS {}!"',
        Rs['character'],
        Rs['character'],
        R('noun', 'noun-mass', 'noun-unique', defaults=[('LIFE', 0.2)],
            postproc=skipart),
        R(('verb-intr', v_cont), defaults=[('CH*RPING', 0.2)])),
    Meme('SEMENCOFFEECANCERHELPIMTRAPPEDINA{}'
            'FACTORYSTARSANDBACONBABIESONICE...',
        R('noun', 'noun-mass', 'noun-unique',
            selector=lambda x: len(x.stringform.split()) == 2,
            postproc=skipart)),
    Meme('SPOILER ALERT! {} {} {} WITH {}!',
        Rs['character'],
        R(('verb-tr', v_present), defaults=[('KILLS', 0.5)]),
        Rs['character'],
        R('noun', 'place')),
    Meme('{} MAKE ME {}',
        C(('MOLPY', 0.1), ('SUDO', 1.0)),
        Rs['noun singular article']),
    Meme('SUP {}, I HERD U DIDN\'T LIEK {}, BUT I ACCIDENTALLY IN YOUR {}.',
        R('character', postproc=dawgify),
        Rs['verb tr cont'],
        Rs['noun singular mass']),
    Meme('THAT\'S {} ASS-{}',
        Rs['adjective article'],
        Rs['noun singular']),
    Meme('THE NICE THING ABOUT HEAD{} IS THAT IT\'S REALLY EASY {} TO '
            'BELIEVE IN THEM',
        R(('noun', n_plural), defaults=[('TREBUCHETS', 0.33),
            ('TREBUCHATS', 0.5)], postproc=aggile),
        C(('OTHER PEOPLE', 0.5), ('OTTERS', 1.0))),
    Meme('TO THE SURPRISE OF {}, {} {} TO BE THE BEST SYSTEM FOR DETERMINING '
            'WHICH OF TWO THINGS IS {}.',
        R(('noun', n_plural), 'character', defaults=[('PUNDITS', 0.25)]),
        R(('noun', n_singular, n_plural), defaults=[('NUMBERS', 0.20)]),
        IndexMap(1, lambda x: 'CONTINUES' if x.originclass == 'noun' \
            and x.wordform == n_singular else 'CONTINUE'),
        R(('adjective', a_comp), defaults=[('LARGER', 0.25)],
            postproc=skipart)),
    Meme('WE HAD A {} PARTY, BUT IT TURNED OUT NOT TO BE VERY {}.',
        Rs['noun mass'],
        R('adjective-positive', defaults=[('MOLPISH', 0.5)],
            postproc=skipart)),
    Meme('WELL, IT\'S MOLPIER THAN {}',
        Rs['noun singular article mass']),
    Meme('WHOA, IT\'S {0} {1}! UNDER {2}\'S SYSTEM, THAT {0} WILL '
            '_NEVER HAPPEN AGAIN!!_',
        C(('FRAME', 0.33), ('NP', 0.5), ('NEWPIX', 1.0)),
        lambda: random.randrange(1000, 3000),
        C(('GLR', 0.25), ('MSCHA', 0.33), ('GEEKWAGON', 0.5),
            ('AUBRONWOOD', 1.0))),
    Meme('WHO WERE YOU, {}? _WHAT DID YOU SEE?!_',
        Rs['character skipthe']),
    Meme('YOU\'RE {}!',
        Rs['noun singular article'])
]

memes_automeme_unmolpish = [
    Meme('BITCHES DON\'T KNOW ABOUT MY {}',
        Rs['noun singular plural mass']),
    Meme('HELLO. MY NAME IS {}. YOU KILLED MY {}. PREPARE TO {}.',
        Rs['character'],
        Rs['noun singular plural mass'],
        R('verb-intr', defaults=[('DIE', 0.9)])),
    Meme('I\'M {0}! I\'M {0}! SUCK MY DIIIIICK! I\'M {0}!',
        Rs['noun singular article character']),
    Meme('IT\'S OVER. {} IS FINISHED.',
        Rs['noun singular mass character property']),
    Meme('{} HAET {}',
        R('character', defaults=[('LOLI', 0.5)]),
        Rs['noun singular plural mass character property']),
    Meme('{} PIG DISGUSTING',
        R(('noun', n_singular, n_plural), 'noun-unique', 'character',
            ('adjective', a_property), postproc=lambda x: ('THE ' \
                if x.originclass == 'noun-unique' else '') + \
                skipart_mup(x) + (' ARE' if x.wordform == 1 else ' IS'))),
    Meme('{} IS THE {} THAT IS CH*RPING {}',
        R('noun', 'noun-unique', 'noun-mass', 'character',
            postproc=lambda x: (random.choice(('THIS ', 'THAT ')) \
                + skipword(x)) if x.originclass == 'noun' \
                else (('THE ' + skipword(x)) \
                    if x.originclass == 'noun-unique' else skipart_mup(x))),
        R('noun', 'noun-unique', 'character',
            defaults=[('BACONSEMENCOFFEEBABYCANCER', 0.8)], postproc=skipart),
        Rs['noun plural mass character']),
    Meme('{} KILLS {}',
        Rs['noun singular character'],
        Rs['noun plural character']),
    Meme('{0} YOUR CH*RPING {1}. {0} THEM.',
        Rs['verb tr inf'],
        Rs['noun plural'])
]

memes_other_unmolpish = [
    Meme('TAKE THIS {} AND CH*RP IT',
        Rs[ 'noun singular mass']), # Credit: mrob27
    Meme('THAT\'S IT! I HAVE HAD IT WITH THESE MOLPYCHIRPING {} ON THIS '
            'MOLPYCHIRPING {}!',
        Rs['noun plural'],
        R('noun', 'noun-unique', defaults=[('RAFTCASTLE', 0.15)],
            postproc=skipart)),
    Meme('WELL CH*RP ME {} WITH {}',
        Rs['adverb'],
        Rs['noun singular article']),
    Meme('{} IS WHY WE CAN\'T HAVE {} THINGS.',
        R('noun', 'noun-unique', 'noun-mass', 'character',
            postproc=lambda x: (random.choice(('THIS ', 'THAT ')) \
                + skipword(x)) if x.originclass == 'noun' \
                else (('THE ' + skipword(x)) \
                if x.originclass == 'noun-unique' else skipart_mup(x))),
        C(('MOLPISH', 0.5), ('NICE', 1.0)))
]

def print_help(f=sys.stdout):
    print("""Usage: automome.py [option] [num]
[option] may be one of the following:
    -h, --help                      show this message and exit
    -V, --version                   show version and exit
    -c [cats], --categories=[cats]  generate memes from the categories [cats]
                                    (see below); the default is aom

[cats] may be any combination of the following letters:
    a       molpish memes from the original AUTOMEME (possibly OTTified)
    o       molpish other memes, i.e. non-AUTOMEME memes
    m       molpish meta-memes, i.e. memes originating in the OTT
    A       unmolpish AUTOMEME memes
    O       unmolpish other memes
    t       OTToMeme memes (xkcd memes by azule)

[num] is the number of memes to generate; if it is missing or invalid, it is
assumed to be 1.""", file=f)

def print_version():
    print("""automome version 4.0, the OTTish AUTOMEME clone
Copyright (C) 2014 Penguin Development
Inspired by AUTOMEME (http://automeme.net/)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.""")

if __name__ == '__main__':
    n = 1
    memes = []
    opts, args = getopt.gnu_getopt(sys.argv[1:], 'hVc:', ('help', 'version',
        'categories='))

    for o, a in opts:
        if o in ('-h', '--help'):
            print_help()
            sys.exit(0)
        elif o in ('-V', '--version'):
            print_version()
            sys.exit(0)
        elif o in ('-c', '--categories'):
            if 'a' in a:
                memes += memes_automeme_molpish
            if 'o' in a:
                memes += memes_other_molpish
            if 'm' in a:
                memes += memes_meta_molpish
            if 'A' in a:
                memes += memes_automeme_unmolpish
            if 'O' in a:
                memes += memes_other_unmolpish
            if 't' in a:
                memes += memes_ottomeme

    if len(args) > 0:
        n = int(args[0])

    if n < 1:
        n = 1

    if len(memes) == 0:
        memes = memes_automeme_molpish + memes_other_molpish + \
                memes_meta_molpish

    for i in range(n):
        print(str(random.choice(memes)))
